import java.io.*;


import PeakProcessing.Output;
import meme.*;
import genome.*;
import ru.genetika.pwm.PwmFormatException;


public class Main {

	public static void main(String[] args) throws IOException, PwmFormatException {
		
		String usage = "java -Xmx1565m -jar PATH_TO_DIR/micsa.jar -f <FindPeaks output file> -n <maximum number of false positives> -g <genome directory> -o <output directory> -l <log file> -w <wif.gz file> -name <optional: name for result file>";
		
		
	
		if (args.length < 12) {
			System.err.println("Too few arguments\n");
			System.err.println(usage);
			System.exit(1);
		}
		
		File inputFile = null;
		File wigFile = null;
		File logFile = null;
		Float FPN = 40f; //maximal false positive number
		File outputDirectory = null;
		File genomeDirectory = null;
		int maxNumberOfBestPeaks = 300;			  //parameter
		float delta = 6f; // delta for best sequences selection
		int currentIndex = 0;
		String outFileName = "selectedPeaks.txt";
		String logFileName = null;

		while (currentIndex<args.length) {
			//System.out.println(args[currentIndex]);	
			if (args[currentIndex].compareTo("-f")==0) {
				currentIndex++;
				inputFile = new File(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-n")==0) {
				currentIndex++;
				FPN = Float.parseFloat(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-o")==0) {
				currentIndex++;
				outputDirectory = new File(args[currentIndex]);				
				if (!outputDirectory.isDirectory()) {
					System.err.println("No such directory: "+outputDirectory);
					System.exit(1);
				}				
			}
			else if (args[currentIndex].compareTo("-l")==0) {
				currentIndex++;
				logFile = new File(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-w")==0) {
				currentIndex++;
				wigFile = new File(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-b")==0) {
				currentIndex++;
				maxNumberOfBestPeaks = Integer.parseInt(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-d")==0) {
				currentIndex++;
				delta = Float.parseFloat(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-name")==0) {
				currentIndex++;
				outFileName = args[currentIndex];
				logFileName = outFileName+".log";
	            SaveOutput.start (logFileName);	
			}
			else if (args[currentIndex].compareTo("-g")==0) {
				currentIndex++;
				genomeDirectory = new File(args[currentIndex]);
				if (!genomeDirectory.isDirectory()) {
					System.err.println("No such directory: "+genomeDirectory);
					System.exit(1);
				}
			}
			currentIndex++;
		}
		
		//System.exit(1);		

		Genome genome = new Genome(genomeDirectory);
		

		//String fileWithChromosomeNames = "list.txt";
		//Genome genome = new Genome(fileWithChromosomeNames); //parameter - file with chromosome names

		//Genome genome = new Genome(genomeDirectory); //parameter - file with chromosome names
		System.out.println("Number of chromosomes read: "+genome.getNumberOfChromosomes());
				
		Output myOutput = new Output(inputFile,logFile, wigFile); 
		genome.setPeakSequences(myOutput);
		
		
		File fileWithBestSeqs = new File (outputDirectory,"bestPeakSeq.txt");
		myOutput.setTotalNumberOfFalsePositives(FPN);
		myOutput.createFileWithBestSeqs(fileWithBestSeqs, maxNumberOfBestPeaks,delta);
		Meme myMeme = new Meme(outputDirectory, fileWithBestSeqs.getName());
		
		
		//FastaText text = new FastaText(fileWithBestSeqs);	 
		double [] letterFreqs = null;
		letterFreqs = genome.getLetterFreqs();
		//System.exit(1);
		myMeme.fillPSSM(letterFreqs);				//rather genome frequencies?
		myMeme.fillThresholds();
		genome.setMotifProbabilities(myMeme.getMotif(0));
		System.out.println(myMeme.getMotif(0).getMinThreshold()+" "+ myMeme.getMotif(0).getProb(myMeme.getMotif(0).getMinThreshold()));
		int numberOfAdditionalMotifs = myMeme.secondRun(genome,letterFreqs);
		for (int i = 1; i<=numberOfAdditionalMotifs; i++) {					
			System.out.println(myMeme.getMotif(i).getMinThreshold()+" "+ myMeme.getMotif(i).getProb(myMeme.getMotif(i).getMinThreshold()));						
		}
		myOutput.runPeakSelection(myMeme.getMotifs());
		File fileWithSelectedPeaks = new File (outputDirectory,outFileName);

		myOutput.printPeaks(fileWithSelectedPeaks);
		
		if (!fileWithBestSeqs.exists())
		      throw new IllegalArgumentException(
		          "Delete: no such file or directory: " + fileWithBestSeqs);

		if (!fileWithBestSeqs.canWrite())
		      throw new IllegalArgumentException("Delete: write protected: "
		          + fileWithBestSeqs);

		    // If it is a directory, make sure it is empty
		if (fileWithBestSeqs.isDirectory()) {
		      String[] files = fileWithBestSeqs.list();
		      if (files.length > 0)
		        throw new IllegalArgumentException(
		            "Delete: directory not empty: " + fileWithBestSeqs);
		}

		    // Attempt to delete it
		boolean success = fileWithBestSeqs.delete();

		if (!success)
		      throw new IllegalArgumentException("Delete: deletion failed");
        SaveOutput.stop (); 

		
	}
}
