package PeakProcessing;


import java.util.ArrayList;
import java.util.HashMap;

import java.util.List;

import ru.genetika.common.CharSequence;

import ru.genetika.pwm.Pwm;
import ru.genetika.pwm.scanner.PwmScannerNaive;

import meme.Motif;
import meme.Sample;
import meme.Site;

public class Optimization {
	private double maxNumberOfFalsePositives;
	private List<Integer> classeslist;
	private HashMap<Integer, PeakClass> peakList;
	private static int numberOfEpsilonN = 1000; // = 100; mirovaya konstanta
	
	public Optimization(List<Integer> classeslist,
			double maxNumberOfFalsePositives,
			HashMap<Integer, PeakClass> peakList) {
	this.maxNumberOfFalsePositives = maxNumberOfFalsePositives;
	this.classeslist = classeslist;
	this.peakList = peakList;	
		
	}
	
	private class SArray {
		float [] arr;
		int size;
		private SArray (int size) {
			this.size = size;
			arr = new float [size];
		}
	}
	
	/*
	private class FArray {
		float [] arr;
		int size;
		private FArray (int size) {
			this.size = size;
			arr = new float [size];
		}
	}
	*/
	
	private class OneClassArrays {
		int size;
		SArray sArray;
		//FArray fArray; + arrays for delta and thresh
		int [] thresholds;
		int [] deltas;
		private OneClassArrays (int size) {
			this.size = size;
			sArray = new SArray(size);
			thresholds = new int [size];
			deltas = new int [size];
			
			for (int j = 0; j < size; j++) {
				sArray.arr[j] = 0;
				thresholds[j] = -1;
				deltas[j] = -1;
			}
			
			//fArray = new FArray(size);
		}
		public void fill(FloatBoard fbForS, FloatBoard fbForF, float [] epsilonArray) {
			int deltIndex = 0;
			int thrIndex = fbForS.thresh.length-1;
			fbForS.uncheck();
			
			for (int deltInd = deltIndex; deltInd<fbForS.delta.length;deltInd++) {
				for (int threshInd = thrIndex; threshInd >= 0; threshInd--) {
					if (!recalc (deltInd,threshInd,fbForS,fbForF,epsilonArray)) {
						threshInd = -1;
					}
				}
			}	
								
		}
		private boolean recalc(int deltIndex, int thrIndex, FloatBoard fbForS,
				FloatBoard fbForF, float[] epsilonArray) {
			
			if ((deltIndex>=fbForS.delta.length)||(thrIndex<0)||(thrIndex>=fbForS.thresh.length)||(deltIndex<0)||(fbForS.board[deltIndex][thrIndex].changed==true)) {
				return false;
			}
			fbForF.board[deltIndex][thrIndex].changed = true;
			int i = place(fbForF.board[deltIndex][thrIndex].value,epsilonArray);
			
			if (i != -1) { // f was placed, which means f <= T
				if (fbForS.board[deltIndex][thrIndex].value > sArray.arr[i]) { //replace S value if this one is greater than S !!!!! greater ot equal? TODO
					sArray.arr[i] = fbForS.board[deltIndex][thrIndex].value;
					thresholds[i] = thrIndex;
					deltas[i] = deltIndex;
				}
				//recalc (deltIndex+1, thrIndex, fbForS,fbForF,epsilonArray);
				//recalc (deltIndex, thrIndex-1, fbForS,fbForF,epsilonArray);
				//recalc (deltIndex+1, thrIndex-1, fbForS,fbForF,epsilonArray);
				return true;
			}		
			return false;
		}
		public int place(float value, float[] epsilonArray) {
			
			if (value<= epsilonArray[0])
				return 0;
			for (int i = 1; i < epsilonArray.length; i++)
			{
				if ((epsilonArray[i-1]<value)&&(value<= epsilonArray[i]))
					return i;
			}
			return -1;
		}
	}
	
	
	public void runOptimization(List <Motif> motifs) {
			
		
		int numberOfMotifs = motifs.size();
		float [] delta = new float[] {0.01f,0.5f,1f,2f,3f,4f,5f}; //mirovye konstanty
		List <float []> thresholds = new ArrayList <float []>();
		float [] minThreshold = new float [numberOfMotifs];
		
		for (int i = 0; i<numberOfMotifs; i++) {
			minThreshold[i] = motifs.get(i).getMinThreshold();
			thresholds.add(motifs.get(i).getThresholdsAsArray());
		}
		
		float [] epsilonArray = new float [numberOfEpsilonN];
		float epsilon = (float) (maxNumberOfFalsePositives/(numberOfEpsilonN-1));
		for (int i = 0; i<numberOfEpsilonN; i++) {
			epsilonArray[i] = i*epsilon;
		}
				
				
		FinalArray fa = new FinalArray(numberOfEpsilonN);
		//fill in floatBoards for each class
		for (int j = 0; j < classeslist.size() ; j++) {
		//for (int j = classeslist.size()-1; j >=0 ; j--) {
			Integer classN = classeslist.get(j);
		//	System.out.print("check "+classN);
			FloatBoard [] fbForS = new FloatBoard [numberOfMotifs];
			FloatBoard [] fbForF = new FloatBoard [numberOfMotifs];
			for (int i = 0; i<numberOfMotifs; i++) {
					
				fbForS[i] = new FloatBoard(thresholds.get(i), delta, classN);
				fbForF[i] = new FloatBoard(thresholds.get(i), delta, classN);
				fbForS[i].fillS(motifs.get(i), peakList.get(classN));
				fbForF[i].fillF(motifs.get(i), peakList.get(classN));
				
				fillFDR(motifs.get(i), peakList.get(classN));
				
				//create array of best S (and threshold and delta) 				
				OneClassArrays oca = new OneClassArrays(numberOfEpsilonN);
				oca.fill(fbForS[i],fbForF[i],epsilonArray);	
				fa.add(oca);
			}
		}
		
		//for (int i = 0; i<numberOfEpsilonN; i++) {
		//	System.out.print(epsilonArray[i]+" "+fa.sumArray[i]+" "+fa.deltaIndices[i] +" "+fa.threshIndices[i]+"\n");
		//}
		String[] treshIndicesStr = fa.threshIndices[numberOfEpsilonN-1].split(",");
		int [] treshIndices = new int[treshIndicesStr.length];
		for (int i = 0; i<treshIndicesStr.length; i++)	{			
			treshIndices[i] = Integer.parseInt(treshIndicesStr[i]);
		}
		String[] deltaIndicesStr = fa.deltaIndices[numberOfEpsilonN-1].split(",");
		int [] deltaIndices = new int[deltaIndicesStr.length];
		for (int i = 0; i<deltaIndicesStr.length; i++)	{			
			deltaIndices[i] = Integer.parseInt(deltaIndicesStr[i]);
		}
		
		for (int j = 0; j < classeslist.size() ; j++) {
			Integer classN = classeslist.get(j);
			float [] selectedDeltas = new float[numberOfMotifs];
			float [] selectedThresholds = new float[numberOfMotifs];

			for (int i = 0; i<numberOfMotifs; i++) {
				int deltInd = deltaIndices[i+j*numberOfMotifs];
				int treshInd = treshIndices[i+j*numberOfMotifs];
				if ((deltInd>=0)&&(deltInd<delta.length))
					selectedDeltas[i] = delta[deltInd];
				else if (deltInd<0)
					selectedDeltas[i] = -1;
				else if (deltInd==delta.length)
					selectedDeltas[i] = classN+1;
				
				if (treshInd>0)
					selectedThresholds[i] = thresholds.get(i)[treshInd];
				else 
					selectedThresholds[i] = -1;
			}
			peakList.get(classN).setSelectedThreshold(selectedThresholds);
			peakList.get(classN).setSelectedDelta(selectedDeltas);
			peakList.get(classN).selectPeaks(motifs);
		}
		

	}
	
	private void fillFDR(Motif motif, PeakClass peakClass) {
		
		float correction = 1;
		if ((peakClass.getSize()>0)&&(peakClass.getFalsePositives()<peakClass.getSize()))
			correction = peakClass.getFalsePositives()/peakClass.getSize();
		
		for (Peak peak : peakClass.getAllPeaks()) {
			
			float maxht = peak.getMaxht();
			
			PwmScannerNaive scanner = new PwmScannerNaive();
			scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));							
			
			scanner.setMinThreshold( motif.getMin_thresh());	
			
			CharSequence seqC = new CharSequence ("", peak.getSequence().toCharArray());				
			scanner.setSequence(seqC);					
				
			Sample sample = scanner.scanToSample();		
			
			float maxScore = 0;
			float minDelta = 100000f;  //big number
			float [] cover = peak.getCoverage();
			
			for (int j = 0; j < sample.getNumberOfSites(); j++) {
				Site site = sample.siteAt(j);	
				float deltaTMP = maxht-peak.getCoverageAt(motif.getWidth()/2+ site.getPosition());
				float scoreTMP = site.getScore();
				int lengths = 0;
				if ((maxScore<scoreTMP)||(minDelta>deltaTMP)) {
					
										
					for (int pos = 0; pos< cover.length; pos++) {						
							if (maxht-cover[pos] <= deltaTMP) {
								lengths++;
							}
					}
					
					float pvalue = FloatBoard.Pvalue(lengths, motif.getProb((float) Math.floor(scoreTMP)));
					pvalue = pvalue*correction;
						
					if (pvalue<peak.getFDR()) {
						peak.setFDR(pvalue);
						minDelta = deltaTMP;
						maxScore=scoreTMP;
					}
						
					
				}
			}
			
				
			
		}
	}

	private class FinalArray {
		float [] sumArray;
		String [] deltaIndices;
		String [] threshIndices;
		int depth;
		public FinalArray(int size) {
			sumArray = new float [size];
			deltaIndices = new String [size];
			threshIndices = new String [size];
			depth = 0;
			
			for (int i = 0; i<size; i++) {
				sumArray[i] = 0f;
				deltaIndices[i] = "";
				threshIndices[i] = "";
			}			
		}
		public void add(OneClassArrays oca) {
			if (depth == 0) {
				depth = 1;
				for (int i = 0; i< oca.size ; i++) {
					sumArray[i] = oca.sArray.arr[i];
					if(sumArray[i] != 0) {
						deltaIndices[i] += oca.deltas[i] + ",";
						threshIndices[i] += oca.thresholds[i] + ",";
					}
					else {
						deltaIndices[i] += "-1,";
						threshIndices[i] += "-1,";
					}
					
				}
				return;
			}
			depth++;
			float[] newSumArray = new float[sumArray.length];
			for (int i = 0; i< oca.size ; i++) {
				int chosen = 0;
				float maxSum = oca.sArray.arr[0] + sumArray[i];
				for (int j = 1; j<=i; j++){ //choose j
					float sum = oca.sArray.arr[j] + sumArray[i-j]; 
					if (sum > maxSum) {
						chosen = j;
						maxSum = sum;
					}
				}
				newSumArray[i] = maxSum;
				if (maxSum!=0){
					deltaIndices[i] += oca.deltas[chosen] + ",";
					threshIndices[i] += oca.thresholds[chosen] + ",";
				}
				else {
					deltaIndices[i] += "-1,";
					threshIndices[i] += "-1,";
				}				
			}
			sumArray = newSumArray.clone();
			
		}		
	}
	
	private static class FloatBoard {
		float [] delta;
		float [] thresh;
		BoardElement [][] board;
		
				
		private FloatBoard ( float [] thresh, float [] delta, Integer classN) {
			this.delta =  new float [delta.length+1] ;
			this.thresh = thresh;	
				
			board = new BoardElement [this.delta.length][thresh.length];
			
			for (int i = 0; i < delta.length; i++) {
				this.delta[i] = delta[i];
				for (int j = 0; j < thresh.length; j++) {
					board[i][j] = new BoardElement();	
				}
			}
			this.delta[delta.length] = classN+1;
			for (int j = 0; j < thresh.length; j++) {
				board[delta.length][j] = new BoardElement();	
			}
			
		}

		private class BoardElement {
			float value;
			boolean changed;
			private BoardElement (){
				value = 0;
				changed = false;
			}
			void check() {
				changed = true;			
			}
			void uncheck() {
				changed = false;	
			}
			public void addOne() {
				if (changed==false) {
					value++;
					changed = true;		
				}
				
			}
			public void add(float pvalue) {
				value += pvalue;	
			}			
		}
		
		public void uncheck () {
			for (int i = 0; i < delta.length; i++) {
				for (int j = 0; j < thresh.length; j++) {
					board[i][j].uncheck();
				}
			}			
		}
		
		public void fillF(Motif motif, PeakClass peakClass) {
			float correction = 1;
			if ((peakClass.getSize()>0)&&(peakClass.getFalsePositives()<peakClass.getSize()))
				correction = peakClass.getFalsePositives()/peakClass.getSize();
			for (Peak peak : peakClass.getAllPeaks()) {
				float maxht = peak.getMaxht();
				float [] cover = peak.getCoverage();
				
				int [] lengths = new int [delta.length];
				
				for (int id = 0; id<delta.length; id++) {
						lengths[id] = 0;					
				}
				
				for (int pos = 0; pos< cover.length; pos++) {
					for (int id = 0; id<delta.length; id++) {
						if (maxht-cover[pos] <= delta[id]) {
							lengths[id]++;
						}
					}
					if (maxht-cover[pos] > delta[delta.length-1]) {
						//System.out.println("This can't be");
						lengths[delta.length-1]++;						
					}
				}
				
				for (int id = 0; id<delta.length; id++) {
					for (int it = 0; it<thresh.length; it++) {
						float pvalue = Pvalue(lengths[id], motif.getProb(thresh[it]));
						board[id][it].add(pvalue*correction);
						//if ((id==5)&&(it==thresh.length-1))
							//System.out.println(board[id][it].value + "\t");
					}				
					//System.out.println("\n");
				}
			}
			
		}
		
		public static float Pvalue(int length, Double prob) {
			return (float) (1-Math.pow(1-prob, length));		
		}

		public void fillS(Motif motif, PeakClass peakClass) {
			for (Peak peak : peakClass.getAllPeaks()) {
			
				float maxht = peak.getMaxht();
				
				PwmScannerNaive scanner = new PwmScannerNaive();
				scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));							
				
				scanner.setMinThreshold( motif.getMin_thresh());	
				
				CharSequence seqC = new CharSequence ("", peak.getSequence().toCharArray());				
				scanner.setSequence(seqC);					
					
				Sample sample = scanner.scanToSample();						
				for (int j = 0; j < sample.getNumberOfSites(); j++) {
					Site site = sample.siteAt(j);					
					int thrIndex = findThreshold(site.getScore());
					int delIndex = findDelta(maxht-peak.getCoverageAt(motif.getWidth()/2+ site.getPosition()));
					for (int id = delIndex; id<delta.length; id++) 
						for (int it = 0; it<=thrIndex; it++) {
							board[id][it].addOne();
						}
					/*
					for (int id = 0; id<=delIndex; id++) //TODO check if not for (int id = delIndex; id<=delta.length; id++)
						for (int it = thrIndex; it<thresh.length; it++) {
							board[id][it].addOne();
						}
					*/
				}
				uncheck ();
			}
			/*
			System.out.println("delta");
			for (int id = 0; id<delta.length; id++) {
				for (int it = 0; it<thresh.length; it++) {
					System.out.println(board[id][it].value);
				}
				System.out.println("\n");
			}
			*/
			
		}

		private int findThreshold(float score) {
			if (score >= thresh[thresh.length-1])
				return thresh.length-1;
			if (score <= thresh[0])
				return 0;
			for (int i = 0; i<thresh.length-1; i++) {
				if ((thresh[i] <= score)&&(score < thresh[i+1])) 
					return i;
			}			
			return -1;
		}

		private int findDelta(float d) {
			if (d >= delta[delta.length-1])
				return delta.length-1;
			if (d <= delta[0])
				return 0;
			for (int i = 0; i<delta.length-1; i++) {
				if ((delta[i] < d)&&(d <= delta[i+1])) 
					return i+1;
			}			
			return -1;
		}
	}
	

}
