package PeakProcessing;

import genome.Genome;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;

import meme.Motif;

public class Output {  //the desired output of FindPeaks
	private HashMap <String, PeaksOnChromosome> peakAtChro;
	//private double totalMaxNumberOfFalsePositives;	// parameter - maximal number of false positives allowed
	
	private List <String> chromosomes;
	private List <Integer> classes;		
	
	public PeaksOnChromosome getPeakAtChro (String chr) {
		if (peakAtChro.containsKey(chr))
			return peakAtChro.get(chr);
		chr = chr.replace("chr", "");
		if (peakAtChro.containsKey(chr))
			return peakAtChro.get(chr);
		chr = "chr"+chr;
		if (peakAtChro.containsKey(chr))
			return peakAtChro.get(chr);
		return null;
	}
	
	public List <String> getAllChromosomes () {
		return chromosomes;
	}
	public List <Integer> getValuesOfClasses() {
		return classes;
	}
	public void setTruePeakSequencesAreSet(String chr) {
		if (peakAtChro.containsKey(chr)){
			 peakAtChro.get(chr).ifPeakSequencesAreSet = true;
			 return;
		}
		chr = chr.replace("chr", "");
		if (peakAtChro.containsKey(chr)){
			 peakAtChro.get(chr).ifPeakSequencesAreSet = true;
			 return;
		}
		chr = "chr"+chr;
		if (peakAtChro.containsKey(chr))
			 peakAtChro.get(chr).ifPeakSequencesAreSet = true;

	}
	
	public boolean isPeakSequencesAreSet(String chr) {
		
		if (peakAtChro.containsKey(chr)){
			 return peakAtChro.get(chr).ifPeakSequencesAreSet;
		}
		chr = chr.replace("chr", "");
		if (peakAtChro.containsKey(chr)){
			 return peakAtChro.get(chr).ifPeakSequencesAreSet;
		}
		chr = "chr"+chr;
		if (peakAtChro.containsKey(chr))
			 return peakAtChro.get(chr).ifPeakSequencesAreSet;
		return false;
	}
	
	private class PeaksOnChromosome {
		private HashMap <Integer, PeakClass> peakList;		
		double maxNumberOfFalsePositives;	// parameter - maximal number of false positives allowed for each chromosome
		int lastClassIncludedToBest;
		String chromName;
		boolean ifPeakSequencesAreSet;
		
		public PeaksOnChromosome () {
			peakList = new HashMap <Integer, PeakClass>();
			ifPeakSequencesAreSet = false;
		}
		public PeaksOnChromosome (String chromName) {
			peakList = new HashMap <Integer, PeakClass>();
			this.chromName = chromName;
			ifPeakSequencesAreSet = false;
		}

		private PeakClass getPeakClass (int i) {
			return peakList.get(i);
		}
		private void setNumberOfFalsePositives(double maxNumberOfFalsePositives) {
			this.maxNumberOfFalsePositives = maxNumberOfFalsePositives;			
		}
		public void setSequences(Genome genome) {
			Iterator<Integer> it = peakList.keySet().iterator();
			while (it.hasNext()) {
				Integer classValue = it.next();
				peakList.get(classValue).setSequences(genome);
			}					
		}
		public void runPeakSelection(List <Motif> motifs) {			
			List <Integer> classeslist = new ArrayList <Integer> ();
			for (Integer i = lastClassIncludedToBest - 1; i>0; i--) {
				if (peakList.containsKey(i)) {
					classeslist.add(i);					
				}				
			}	
			System.out.println("Running peak selection procedure on "+chromName);
			Optimization myOptimization = new Optimization(classeslist, maxNumberOfFalsePositives, peakList);
			myOptimization.runOptimization(motifs);
		}

		public Peak find(Integer start) {
			for (Integer i : classes) {
				PeakClass pk = peakList.get(i);
				for (Peak peak :pk.getAllPeaks()) {
					if (Math.abs(start - peak.getPeakStart())<2) {
						return peak;
					}
				}
			}
			return null;
		}
	}
	
	public Output(File inputFile, File logFile, File wigFile) {
		peakAtChro = new HashMap <String, PeaksOnChromosome>();
		chromosomes = new ArrayList <String> ();
		classes = new ArrayList <Integer> ();
		
		try {
			System.out.println("Reading log file");
			readLogFile(logFile);
			System.out.println("Reading file with peaks");
			readPeakFile(inputFile);
			System.out.println("Reading wig file");
			readWigFile(wigFile);
			System.out.println("wig file read");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	public void readWigFile (File wigFile) throws IOException {
		
		if (!wigFile.getName().matches(".*gz")) {
			System.err.println("wig file must be gzipped");
        	System.exit(1);
		}
		BufferedReader br = new BufferedReader(new InputStreamReader(new GZIPInputStream(new FileInputStream(wigFile))));	
		String strLine;
		Pattern pattern = Pattern.compile( ".*chrom=(\\S+)\\s+start=(\\d+)\\s.*");
		Pattern patternNumber = Pattern.compile( "^\\d+\\.*\\d*E?-?\\d*"); //8.00E-4
		Peak peak = null;
		int i = 0;
		float [] heights = null;
		 try {
				while ((strLine = br.readLine()) != null)   {
					//System.out.println(strLine); 
					Matcher matcher = pattern.matcher( strLine );
					if (matcher.find()) {	
						//System.out.println(strLine); 
						String chrom = matcher.group(1).replaceAll(".fa", "");
						Integer start = Integer.parseInt(matcher.group(2));
						if(getPeakAtChro(chrom)==null) 
							peak = null;						
						else
							peak = getPeakAtChro(chrom).find(start);
						if (peak!= null) {
							heights = new float [peak.getSequenceLength()]; 
							//System.out.println(peak.getPeakStart()); 
						}
					}
					if (peak!= null) {						
						matcher = patternNumber.matcher( strLine );
						if (matcher.find()) {	
							Float height = Float.parseFloat(matcher.group(0));
							heights[i] = height;
							i++;
						}
						if (i == heights.length) {
							peak.setCoverage(heights);
							i = 0;
						}
						
					}
				}
		} catch (IOException e) {
				System.err.println("Error: " + e.getMessage());
				e.printStackTrace();
		}
		
	}
	
	public void readPeakFile (File peakFile) throws IOException {
		 DataInputStream in = new DataInputStream(new FileInputStream(peakFile));
		 BufferedReader br = new BufferedReader(new InputStreamReader(in));
		 String strLine;
		 
		 Pattern pattern = Pattern.compile( "\\d+\\s+(\\S+)\\s+(\\d+)\\s(\\d+)\\s+\\d+\\s+(\\d+\\.*\\d*).*");
		 
		 	 
		 //Read File Line By Line 

		 try {
			while ((strLine = br.readLine()) != null)   {
			    Matcher matcher = pattern.matcher( strLine );
				
				if (matcher.find()) {								
					//System.out.println(matcher.group(0)+" "+matcher.group(1)+" "+ matcher.group(2)+" "+ matcher.group(3)+" "+ matcher.group(4));
					
					Float peakHeight = Float.parseFloat(matcher.group(4));
					
					Integer classValue = Math.round(peakHeight);
					//System.out.println(classValue);
					String chrom = matcher.group(1).replaceAll(".fa", "");
					if(!chrom.matches("chr.*")) {
						chrom = "chr"+chrom;
					}
					//System.out.println(chrom);
					Integer start = Integer.parseInt(matcher.group(2));
					Integer end = 	Integer.parseInt(matcher.group(3));
					
					if (classValue>0){
					    if (!classes.contains(classValue)) {
					    	classValue = classes.get(classes.size()-1);			    	
					    }						   
					    if (peakAtChro.containsKey(chrom)) {				    	
					        if (!peakAtChro.get(chrom).peakList.containsKey(classValue)) {
					        	System.err.println("Error in Log file");
					        	System.exit(1);
						    	//PeakClass pk = new PeakClass(classValue,chrom);
						    	//peakAtChro.get(chrom).peakList.put(classValue, pk);
						    }					    	
						    peakAtChro.get(chrom).peakList.get(classValue).addPeakAt(chrom, start, end, peakHeight);
					    }
					    else {
					    	String chomosome = chrom.replace("chr", "");
							if (peakAtChro.containsKey(chomosome))	{				    	
							        if (!peakAtChro.get(chomosome).peakList.containsKey(classValue)) {
							        	System.err.println("Error in Log file");
							        	System.exit(1);
								    	//PeakClass pk = new PeakClass(classValue,chrom);
								    	//peakAtChro.get(chrom).peakList.put(classValue, pk);
								    }					    	
								    peakAtChro.get(chomosome).peakList.get(classValue).addPeakAt(chrom, start, end, peakHeight);
							 }							
					    }
					}	
				}	
			}
		} catch (IOException e) {
			System.err.println("Error: " + e.getMessage());
			e.printStackTrace();
		}
		//Close the input stream
		in.close();
		
	}
	
	public void readLogFile (File logFile) throws IOException {
			
		 DataInputStream in = new DataInputStream(new FileInputStream(logFile));
		 BufferedReader br = new BufferedReader(new InputStreamReader(in));
		 String strLine;
		 
		 Pattern pattern = Pattern.compile( "Info:\\s+(\\d)+.*");
		 Pattern patternClass = Pattern.compile( "\\d+\\.*\\d*");
		 
		 String chromosomeName = "";
		 
		 //Float prevValueforControl = 0f;
		 //Float currValueforControl = 0f;

		 
		 //Read File Line By Line
		 try {
			while ((strLine = br.readLine()) != null)   {
				if (strLine.contains("Current chromosome")) {	
						chromosomeName = strLine.replaceAll("(Info:    Current chromosome : )|(.fa)|(\t)", "");	
						if (!peakAtChro.containsKey(chromosomeName)) {
							chromosomes.add(chromosomeName);
							PeaksOnChromosome poc = new PeaksOnChromosome(chromosomeName);
							peakAtChro.put(chromosomeName,poc);
						}
				}				
				else if (strLine.matches("Info:\\s+(\\d)+.*")) {								
					    Matcher matcher = pattern.matcher( strLine );
					    matcher = patternClass.matcher( strLine );
					    					    
					    matcher.find();
					    matcher.find();
					    Integer classValue = (new Float(Float.parseFloat(matcher.group(0)))).intValue();
					    matcher.find();
					    Float sampleN = Float.parseFloat(matcher.group(0));
					    matcher.find();
					    matcher.find();
					    Float normalizControlN = Float.parseFloat(matcher.group(0));
					    if (normalizControlN==0) {
					    	normalizControlN  = new Float( Math.pow(10, -1)); //= (1f/10f/classValue);
					    	//Float diff = prevValueforControl-currValueforControl;
					    	//Float currentDiff = diff*0.5f;
					    	//normalizControlN = currValueforControl-currentDiff;
					    	//System.out.println("prevValueforControl "+prevValueforControl+" currValueforControl "+currValueforControl+" diff "+diff+" normalizControlN "+normalizControlN);
					    }
					   // prevValueforControl = currValueforControl;
					    //currValueforControl = normalizControlN;
					    if (classValue>0){
					    	if (!classes.contains(classValue)) {
					    		classes.add(classValue);
					    	}					    	
					    	if (!peakAtChro.get(chromosomeName).peakList.containsKey(classValue)) {
					    		PeakClass pk = new PeakClass(classValue,chromosomeName);
					    		peakAtChro.get(chromosomeName).peakList.put(classValue, pk);
					    	}					    	
					    	peakAtChro.get(chromosomeName).peakList.get(classValue).setFalsePositives(normalizControlN);
					    	peakAtChro.get(chromosomeName).peakList.get(classValue).setNumberWOselection(sampleN);
					    }
				}	
			}
		} catch (IOException e) {
			System.err.println("Error: " + e.getMessage());
			e.printStackTrace();
		}
		//Close the input stream
		in.close();
		    
		
	}

	public PeakClass getPeakClass(String chomosome, int i) {
		if (peakAtChro.containsKey(chomosome))
			return peakAtChro.get(chomosome).getPeakClass(i);
		else {
			chomosome = chomosome.replace("chr", "");
			if (peakAtChro.containsKey(chomosome))
				return peakAtChro.get(chomosome).getPeakClass(i);
			chomosome = "chr"+chomosome;
			if (peakAtChro.containsKey(chomosome))
				return peakAtChro.get(chomosome).getPeakClass(i);
		}
		return null;
		
	}
	
	public void setTotalNumberOfFalsePositives(double totalMaxNumberOfFalsePositives) {
		//this.totalMaxNumberOfFalsePositives = totalMaxNumberOfFalsePositives;
		for (String chromosome : peakAtChro.keySet()) {			
			peakAtChro.get(chromosome).setNumberOfFalsePositives(totalMaxNumberOfFalsePositives/peakAtChro.size());
		}
	}
	public void setSequences (Genome genome) {
		Iterator<String> it = peakAtChro.keySet().iterator();
		while (it.hasNext()) {
			String chromosome = it.next();
			peakAtChro.get(chromosome).setSequences(genome);
		}		
	}
	
	public int createFileWithBestSeqs(File fileWithBestSeqs, int maxNumberOfPeaks, float delta) throws IOException { //write into a file sequences to be used for motif discovering
		
		FileWriter fstream = new FileWriter(fileWithBestSeqs);
        BufferedWriter out = new BufferedWriter(fstream);		
		int numberSelectedPeaks = 0;
		Integer classValue = classes.get(classes.size()-1);
		
		for (String chromosome: chromosomes) {
			peakAtChro.get(chromosome).lastClassIncludedToBest = classValue+1;
		}
		int lastClassIncludedToBestforAll = classValue+1;
		boolean ifRichedFull = false;
		/*

		for (int i = classes.size()-1; i>=0; i--) {
			//System.out.println("class" + i);
			if (ifRichedFull==false)
				for (String chromosome: chromosomes) {
					classValue = classes.get(i);	
					//System.out.println(i+" "+classValue+" "+chromosome);
	
					PeakClass pc = getPeakClass(chromosome,classValue);
					int numberOfPeaksInThisClass = pc.getSize();
					float numberFP = pc.getFalsePositives();
					if (numberFP <= 2.) //mirovaya konstanta 2!!!
						if (numberSelectedPeaks+numberOfPeaksInThisClass <= maxNumberOfPeaks) {
							numberSelectedPeaks += numberOfPeaksInThisClass;
							out.write(pc.getsequencesAsString(delta));	
							pc.includeToBest();
							peakAtChro.get(chromosome).lastClassIncludedToBest = classValue;
							//if (chromosome.contains("19")==true)
								//System.out.println("19 : "+peakAtChro.get(chromosome).lastClassIncludedToBest);
							if (lastClassIncludedToBestforAll > classValue)
								lastClassIncludedToBestforAll = classValue;
						}
						else {
							ifRichedFull = true;
						}
				}
			
		} //--> replaced below
		
		*/
		
		for (String chromosome: chromosomes) {

			for (int i = classes.size()-1; i>=0; i--){
				//System.out.println("class" + i);

				if (ifRichedFull==false) {

					classValue = classes.get(i);	
					
					System.out.println("setting best sequences for "+chromosome+" class "+classValue);

					//System.out.println(i+" "+classValue+" "+chromosome);
	
					PeakClass pc = getPeakClass(chromosome,classValue);
					int numberOfPeaksInThisClass = pc.getSize();
					float numberFP = pc.getFalsePositives();
					if (numberFP <= 0.5) {//mirovaya konstanta 1!!!
						if (numberSelectedPeaks+numberOfPeaksInThisClass <= maxNumberOfPeaks) {
							numberSelectedPeaks += numberOfPeaksInThisClass;
							out.write(pc.getsequencesAsString(delta));	
							//pc.includeToBest();
							//peakAtChro.get(chromosome).lastClassIncludedToBest = classValue;
							
							//if (chromosome.contains("19")==true)
								//System.out.println("19 : "+peakAtChro.get(chromosome).lastClassIncludedToBest);
							//if (lastClassIncludedToBestforAll > classValue)
							//	lastClassIncludedToBestforAll = classValue;
						}
						else {
							ifRichedFull = true;
						}
					}
					else  {
						i =-1;
					}
				}
			}
		}
		
		/*
		
		for (int i = classes.size()-1; i>=0; i--) {  //don't need it if createLog.pl was used
			classValue = classes.get(i);
			if (classValue >= lastClassIncludedToBestforAll)
			for (String chromosome: chromosomes) {
				PeakClass pc = getPeakClass(chromosome,classValue);
				pc.includeToBest();
				peakAtChro.get(chromosome).lastClassIncludedToBest = lastClassIncludedToBestforAll;
			}
		}
		*/
		out.close();		
		return numberSelectedPeaks;
	}

	public void runPeakSelection(List <Motif> motifs) {
		for (PeaksOnChromosome poc : peakAtChro.values()) {			
			poc.runPeakSelection(motifs);
		}		
	}

	public int printPeaks(File fileWithSelectedPeaks) throws IOException {
		
		FileWriter fstream = new FileWriter(fileWithSelectedPeaks);
        BufferedWriter out = new BufferedWriter(fstream);		
		int numberSelectedPeaks = 0;
		Integer classValue = classes.get(classes.size()-1);
			
		for (int i = classes.size()-1; i>=0; i--) {
			for (String chromosome: chromosomes) {
				classValue = classes.get(i);
				PeakClass pc = getPeakClass(chromosome,classValue);
				float fdr = pc.getFalsePositives();
				//int ifFDRequalZero = 1;
				//if (fdr==0)
				//	ifFDRequalZero = 0;
				for (Peak peak:pc.getAllPeaks()) {
					if (peak.isSelected()){
						out.write(peak.asString(fdr));	
						numberSelectedPeaks++;
					}
				}
			}
			
		}
		out.close();		
		return numberSelectedPeaks;

	}

	public void removePeaksAtChromosome(String chromosomeToDelete) {
		peakAtChro.remove(chromosomeToDelete);
		
		for (String chr : chromosomes) {
			if (chr==chromosomeToDelete) {
				chromosomes.remove(chr);
				return;
			}			
		}	
		
	}	
}
