package PeakProcessing;

public class Peak {
	private float maxht; 				/* Maximal height */
	private String chromosome;			/* Chromosome */
	private int peakStart;				/* Peak Start */
	private int peakEnd;				/* Peak Start */
	private float [] coverageHeights;	/* Coverage along the peak */
	private String sequence;			/* DNA sequence of peak; is calculated in 'Genome' package */
	private int sequenceLength;			/* Peak length */
	private boolean isSelected;			/* is Peak is selected  */
	private float fdr;					/* fdr */
	
	public Peak(float maxht, String chromosome, int peakStart, int peakEnd, float [] coverageHeights) {
		this.maxht = maxht;
		this.chromosome = chromosome;
		this.peakStart = peakStart;
		this.coverageHeights = coverageHeights;
		this.sequenceLength = peakEnd-peakStart+1;
		this.peakEnd = peakEnd;
		isSelected = false;
		fdr = 1f;
	}
	public Peak(float maxht, String chromosome, int peakStart, int peakEnd) {
		this.maxht = maxht;
		this.chromosome = chromosome;
		this.peakStart = peakStart;
		//this.coverageHeights = coverageHeights;
		this.sequenceLength = peakEnd-peakStart+1;
		this.peakEnd = peakEnd;
		coverageHeights = null;
		isSelected = false;
		fdr = 1f;
	}
	
	public void selectPeak() {
		isSelected = true;
	}
	public void setFDR(float fdr) {
		this.fdr = fdr;
	}
	public float getFDR() {
		return fdr;
	}
	public boolean isSelected() {
		return this.isSelected;
	}
	public void setSequence (String s) {  
		sequence = new String(s);
	}
	public String getSequence() {
		return sequence;
	}
	public String getSequence(float delta) {
		String s = "";
		float threshold = maxht-delta;
		for (int i = 0;i<sequenceLength;i++) {
			if (coverageHeights[i]>=threshold) {
				s = s + sequence.charAt(i);
			}
		}
		//System.out.println(s);
		return s;
	}
	public int getSequenceLength() {
		return sequenceLength;
	}
	public String getStringGivenDelta (float delta) {   /* returns a substring with coverage ">=(maxht-delta)"  */
		String s = new String ("");
		for (int i = 0; i<sequenceLength;i++) {
			if (coverageHeights[i]>=maxht-delta)
				s = s + sequence.charAt(i);
		}
		return s;
	}
	public String getChromosome() {		
		return chromosome;
	}
	public int getPeakStart() {
		return peakStart;
	}
	public float getCoverageAt(int i) {
		return coverageHeights[i];
	}
	public float [] getCoverage() {
		return coverageHeights;
	}
	public float getMaxht() {		
		return maxht;
	}
	public void setCoverage(float[] heights) {
		coverageHeights = heights;
		
	}
	public int getPeakEnd() {		
		return peakEnd;
	}
	public String asString(float fdr2) {
		if ((fdr==1)&&(fdr2<0.3)) {
			fdr = 0;
		}
		String str = chromosome+ "\t"+peakStart+"\t"+peakEnd+"\t"+maxht+"\t"+sequenceLength+"\t"+fdr+"\n";
		return str;
	}
	
}
