package PeakProcessing;

import genome.Genome;

import java.util.ArrayList;
import java.util.List;

import ru.genetika.common.CharSequence;
import ru.genetika.pwm.Pwm;
import ru.genetika.pwm.scanner.PwmScannerNaive;

import meme.Motif;
import meme.Sample;
import meme.Site;


/* Class which holds a list of peaks for a given chromosome and DNA fragment coverage */

public class PeakClass {
	private int classValue; 		/* DNA fragment coverage; is equal to 'round(maximalPeakHeight)' for peaks in the list */
	private List <Peak> peaks;		/* List of Peaks */
	private float falsePositives;	/* Number of false positives for a given chromosome and DNA fragment coverage */
	private float peakNumberWOselection;	/* Number of peaks for a given chromosome and DNA fragment coverage without selection */

	private String chromosomeName;	/* Chromosome Name */

	
	/* Results from the optimization */
	private boolean isIncludedToBest;
	private float[] selectedThreshold;  	//PSSM thresholds optimized separately for each motif  
	private float[] selectedDelta;			//delta optimized separately for each motif
	private List <Integer> selectedIndexes; //indices of selected peaks   //TODO mozhet on ne nuzhen?
	
	public void setSelectedThreshold(float[]selectedThreshold) {
		this.selectedThreshold = selectedThreshold;
	}
	public void setSelectedDelta(float[]selectedDelta) {
		this.selectedDelta = selectedDelta;
	}
	
	public PeakClass () {
		isIncludedToBest = false;
		peaks = new ArrayList <Peak>();
		selectedIndexes = new ArrayList <Integer>();
	}
	public PeakClass (int classValue, String chromosomeName) {
		this();
		this.classValue = classValue;
		this.chromosomeName = chromosomeName;
	}
	public int getClassValue() {
		return classValue;		
	}
	public float getFalsePositives() {
		return falsePositives;		
	}
	public void setFalsePositives(float fp) {
		falsePositives = fp;		
	}
	public float getNumberWOselection() {
		return peakNumberWOselection;		
	}
	public void setNumberWOselection(float number) {
		peakNumberWOselection = number;		
	}
	public String getChromosomeName() {
		return chromosomeName;		
	}
	public int getSize() {
		return peaks.size();		
	}
	public Peak getPeakAt (int i) {
		return peaks.get(i);
	}
	public void addPeakAt (String chr, int start, int end, Float height) {
		/*
		for (int i = 0; i<peaks.size();i++) {
			if (end <= peaks.get(i).getPeakStart()) {
				peaks.add(i,new Peak(height, chr, start, end));
				return;
			}
		
		}
				peaks.add(new Peak(height, chr, start, end));		

		*/
		for (int i = peaks.size()-1; i>=0; i--) {
			if (start > peaks.get(i).getPeakEnd()) {
				peaks.add(i+1,new Peak(height, chr, start, end));
				return;
			}
		}				
		peaks.add(0,new Peak(height, chr, start, end));		
	}
	public void setSequences(Genome genome) {
		for (int i = 0; i < peaks.size(); i++)
			genome.setPeakSequences(peaks.get(i));		
	}
	public String getsequencesAsString() {
		String s = "";
		for (int i = 0; i < peaks.size(); i++) {
			s = s + ">" + getPeakAt(i).getChromosome() +":"+ getPeakAt(i).getPeakStart()+ "-"+getPeakAt(i).getPeakEnd()+"\n" + getPeakAt(i).getSequence()+ "\n";
		}
		return s;
	}
	public String getsequencesAsString(float delta) {
		String s = "";
		for (Peak peak : peaks)	{
			s = s + ">" + peak.getChromosome() +":"+ peak.getPeakStart()+ "-"+peak.getPeakEnd()+"\n" + peak.getSequence(delta)+ "\n";
		}
		return s;
	}

	
	public void includeToBest () {
		isIncludedToBest = true;
		int i = 0;
		for (Peak peak : peaks) {
			peak.selectPeak();
			selectedIndexes.add(i);
			i++;
		}
		
	}
	public boolean isIncludedToBest() {
		return isIncludedToBest;
	}
	public List <Peak> getAllPeaks() {
		return peaks;
	}
	public void selectPeaks(List<Motif> motifs) {
		
		for (int i = 0; i<motifs.size();i++) {
			Motif motif = motifs.get(i);
			float thresh = selectedThreshold[i];
			float delta = selectedDelta[i];
			int PeakInd = 0;
			for (Peak peak : peaks) {
				
				float maxht = peak.getMaxht();
				
				PwmScannerNaive scanner = new PwmScannerNaive();
				scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));							
				
				scanner.setMinThreshold( motif.getMin_thresh());	
				
				CharSequence seqC = new CharSequence ("", peak.getSequence().toCharArray());				
				scanner.setSequence(seqC);					
					
				Sample sample = scanner.scanToSample();						
				for (int j = 0; j < sample.getNumberOfSites(); j++) {
					Site site = sample.siteAt(j);					
					if (site.getScore()>=thresh)
						if (maxht-peak.getCoverageAt(motif.getWidth()/2+ site.getPosition()) <= delta) 
							if (!peak.isSelected()){
								peak.selectPeak();
								selectedIndexes.add(PeakInd);
							}
					
				}
				PeakInd++;
				
			}
		}
	}
	public void remove(Peak peak) {
		peaks.remove(peak);
		peakNumberWOselection--;			
	}
	
	
}
