package components;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.krysalis.jcharts.chartData.ChartDataException;

import charts.BarCharts;
import micsa.fun.*;

import micsa.util.ChromosomeNamesComparator;

import java.beans.*;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FilenameFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Random;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ProgressMonitorMicsa extends JPanel
                                 implements ActionListener,
                                            PropertyChangeListener {

	static ResourceBundle res = ResourceBundle.getBundle("micsa");
	
    private ProgressMonitor progressMonitor;
    private JButton startButton;
    private JTextArea taskOutput;
    private FPrunTask fPrunTask;

    private String aligner;
    private File chipFile; 
    private File controlFile;
    private File genomeDir;
    private File outputDir; 
    private Integer minLength;
    private Integer maxLength; 
    private Integer meanLength;
    private int readNumberChip = 0; 
    private int readNumberControl = 0;
	private Integer maqQT;
	private File genomeBlackList;
	private Float ratioForFiltering;
    private String chipOutFilename = "chip_triangle_standard.peaks";
    private String controlOutFilename = "control_triangle_standard.peaks";
    private String summaryFilename ="FindPeaksSummary.txt";
    private String micsaOutputFilename = "micsaPeakSelection.txt";
	private Integer minimalCoverage;
    private Float fpNumber;
    private ArrayList <ArrayList<String>> motifs;

    
    class Task extends SwingWorker<Void, Void> {
        @Override
        public Void doInBackground() {
            Random random = new Random();
            int progress = 0;
            setProgress(0);
            try {
                Thread.sleep(1000);
                while (progress < 100 && !isCancelled()) {
                    //Sleep for up to one second.
                    Thread.sleep(random.nextInt(1000));
                    //Make random progress.
                    progress += random.nextInt(10);
                    setProgress(Math.min(progress, 100));
                }
            } catch (InterruptedException ignore) {}
            return null;
        }

        @Override
        public void done() {
            Toolkit.getDefaultToolkit().beep();
            startButton.setEnabled(true);
            progressMonitor.setProgress(0);
        }
    }
    class FPrunTask extends SwingWorker<Void, Void> {
        
    	@Override
        public Void doInBackground() {
            Random random = new Random();
            int progress = 1;
            setProgress(1);
           String fpRunCommandChip = null;
           String fpRunCommandControl = null;

           if (aligner.equals(res.getString("AL_ELAND"))||aligner.equals(res.getString("AL_ELAND_EXT"))) { 
        	   try {
        		   String alignerType = null;
          		   if (aligner.equals(res.getString("AL_ELAND"))) {
          			 alignerType = "eland";
                    }                	   
                   if (aligner.equals(res.getString("AL_ELAND_EXT"))) {
            			 alignerType = "elandext";
                   }        		   
        		   //create directories to store .part.gz files
        		   File dirForChipData = new File(outputDir,"ChIP");
        		   File dirForControlData = new File(outputDir,"Control");

        		   if (!dirForChipData.exists()) { //create it
        				 boolean success = dirForChipData.mkdir();
        				 if (!success)
        				 	throw (new IOException());
        		   }	
        		   if (!dirForControlData.exists()) { //create it
      				 boolean success = dirForControlData.mkdir();
      				 if (!success)
      				 	throw (new IOException());
        		   }	
        		   //grep chip file
        		   File newFilewithChip = new File(outputDir,chipFile.getName());
        		   String command = "grep§U[012]§"+chipFile.getCanonicalPath();
        		   chipFile = newFilewithChip;        		  
        		   
        		   ProcessBuilder pb = new ProcessBuilder(command.split("§"));
        		   //pb.redirectErrorStream( true );
        		   Process process = pb.start();

        		   BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));        		   
        		   BufferedWriter out = new BufferedWriter(new FileWriter(chipFile));        	       
        		   String s = results.readLine();   
        		   while ((s = results.readLine())!=null) {
        			   out.write(s+"\n"); 
        			   //readNumberChip++;
        		   }
        		   out.close();
        		   progress = 5; setProgress(5);
                   
        		   //grep control file
           		   File newFilewithControl = new File(outputDir,controlFile.getName());
        		   command = "grep§U[012]§"+controlFile.getCanonicalPath();
        		   controlFile = newFilewithControl;
        		   process = new ProcessBuilder(command.split("§")).start();
        		   out = new BufferedWriter(new FileWriter(controlFile));    
        		   results = new BufferedReader (new InputStreamReader(process.getInputStream()));
        		   s = results.readLine();  
        		   while ((s = results.readLine())!=null) {
        			   out.write(s+"\n");     
        			   //readNumberControl++; 
        		   }
        		   out.close();
        		   progress = 10; setProgress(10);       
                   
        		   //separate eland file for chip
	        	   String [] args = new String[3];
	     	  	   args[0] = alignerType;  args[1] = chipFile.getCanonicalPath(); args[2] = dirForChipData.getCanonicalPath();
	     	  	   command = "java§-Xmx1G§-jar§SeparateReads.jar§"+ args[0]+"§"+args[1] +"§"+args[2];
	     	  	   process = new ProcessBuilder(command.split("§")).start();
	     	  	   results = new BufferedReader (new InputStreamReader(process.getInputStream()));        		   
	     	  	   while ((s = results.readLine())!=null) {
	     	  	   }
        		   progress = 15; setProgress(15);   
        		   
          		   //separate eland file for control	        	   
	     	  	   args[1] = controlFile.getCanonicalPath(); args[2] = dirForControlData.getCanonicalPath();
	     	  	   command = "java§-Xmx1G§-jar§SeparateReads.jar§"+ args[0]+"§"+args[1] +"§"+args[2];
	     	  	   process = new ProcessBuilder(command.split("§")).start();
	     	  	   System.out.println(command.replace("§", " "));
	     	  	   results = new BufferedReader (new InputStreamReader(process.getInputStream()));        		   
	     	  	   while ((s = results.readLine())!=null) {
	     	  	   }	        	  // s = console.readLine(); 
        		   progress = 20; setProgress(20);
        		   
        		   //sort eland file for chip	        	   
	     	  	   File [] fileList = dirForChipData.listFiles();
	     	  	   ArrayList<String> chipFnames = new  ArrayList<String>();
	     	  	   for (File f : fileList) {
	     	  		   String fn = f.getCanonicalPath();
	     	  		   if (fn.contains("part.eland.gz"))
	     	  			chipFnames.add(fn);
	     	  	   }
	     	  	   ArrayList<String> comLine = new ArrayList<String> (chipFnames);
	     	  	   comLine.add(0, dirForChipData.getCanonicalPath());
	     	  	   comLine.add(0, alignerType);
	     	  	   comLine.add(0, "SortFiles.jar");
	     	  	   comLine.add(0, "-jar");
	     	  	   comLine.add(0, "-Xmx1G");
	     	  	   comLine.add(0, "java");
	     		   pb = new ProcessBuilder(comLine);
        		   process = pb.start();
	     	  	   results = new BufferedReader (new InputStreamReader(process.getInputStream()));        		   
	     	  	   while ((s = results.readLine())!=null) {
	     	  		  // System.out.println(s);
	     	  	   }
        		   progress = 22; setProgress(22);   
        		   
          		   //sort eland file for control	
        		   fileList = dirForControlData.listFiles();
	     	  	   ArrayList<String> controlFnames = new  ArrayList<String>();
	     	  	   for (File f : fileList) {
	     	  		   String fn = f.getCanonicalPath();
	     	  		   if (fn.contains("part.eland.gz"))
	     	  			controlFnames.add(fn);
	     	  	   }
	     	  	   comLine = new ArrayList<String> (controlFnames);
	     	  	   comLine.add(0, dirForControlData.getCanonicalPath());
	     	  	   comLine.add(0, alignerType);
	     	  	   comLine.add(0, "SortFiles.jar");
	     	  	   comLine.add(0, "-jar");
	     	  	   comLine.add(0, "-Xmx1G");
	     	  	   comLine.add(0, "java");
	     		   pb = new ProcessBuilder(comLine);
        		   process = pb.start();
	     	  	   results = new BufferedReader (new InputStreamReader(process.getInputStream()));        		   
	     	  	   while ((s = results.readLine())!=null) {
	     	  		   System.out.println(s);
	     	  	   }
	     	  	   progress = 25; setProgress(25);  
	     	  	    	   
        		   //Create command for FindPeaks

               	   fpRunCommandChip    = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§"+alignerType+"§-eff_frac§0.7§-duplicatefilter§-input§";
               	   for (String fn : chipFnames) {
               		   fpRunCommandChip += fn+"§";
               	   }              
               	   fpRunCommandChip += "-name§chip§-output§"+outputDir.getCanonicalPath()+"§-minimum§3§-dist_type§1"; 
            	   
               	   fpRunCommandControl = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§"+alignerType+"§-eff_frac§0.7§-duplicatefilter§-input§";
               	   for (String fn : controlFnames) {
               		   fpRunCommandControl += fn+"§";
            	   } 
               	   fpRunCommandControl +=  "§-name§control§-output§"+outputDir.getCanonicalPath()+"§-minimum§1§-dist_type§1"; //
                                 	   
                     
        	   } catch (IOException e) {
						e.printStackTrace();

        	   }    
           }    
           if (aligner.equals(res.getString("AL_MAQ64"))) {
        	   try {         		   //Create command for FindPeaks
	        	   fpRunCommandChip    = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§maq§-maq_read_size§64§-qualityfilter§"+maqQT+"§-eff_frac§0.7§-duplicatefilter§-input§"+chipFile.getCanonicalPath()   +"§-name§chip§-output§"   +outputDir.getCanonicalPath()+"§-minimum§3§-dist_type§1"; //
	    		   fpRunCommandControl = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§maq§-maq_read_size§64§-qualityfilter§"+maqQT+"§-eff_frac§0.7§-duplicatefilter§-input§"+controlFile.getCanonicalPath()+"§-name§control§-output§"+outputDir.getCanonicalPath()+"§-minimum§1§-dist_type§1"; //
        	   }catch (IOException e) {
					e.printStackTrace();
        	   } 
           }
           if (aligner.equals(res.getString("AL_MAQ128"))) {
           	   try {         		   //Create command for FindPeaks
	        	   fpRunCommandChip    = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§maq§-maq_read_size§128§-qualityfilter§"+maqQT+"§-eff_frac§0.7§-duplicatefilter§-input§"+chipFile.getCanonicalPath()  +"§-name§chip§-output§"   +outputDir.getCanonicalPath()+"§-minimum§3§-dist_type§1"; //
	    		   fpRunCommandControl = "java§-Xmx1G§-jar§FindPeaks.jar§-aligner§maq§-maq_read_size§128§-qualityfilter§"+maqQT+"§-eff_frac§0.7§-duplicatefilter§-input§"+controlFile.getCanonicalPath()+"§-name§control§-output§"+outputDir.getCanonicalPath()+"§-minimum§1§-dist_type§1"; //
        	   }catch (IOException e) {
					e.printStackTrace();
        	   } 
           }
           if (aligner.equals(res.getString("AL_BED"))) {
           		return null; //TODO
           }   
           if (aligner.equals(res.getString("AL_MAPVIEW"))) {
          		return null; //TODO
          }   if (aligner.equals(res.getString("AL_GFF"))) {
          		return null; //TODO
          }  
           
		   if (meanLength!=null && meanLength>0) {
			   fpRunCommandChip += "§"+meanLength;
			   fpRunCommandControl += "§"+meanLength;
		   }
  		   if (minLength!=null && maxLength!=null &&minLength>0&&maxLength>0&&meanLength>0) {
			   fpRunCommandChip += "§"+maxLength+"§"+minLength;
			   fpRunCommandControl += "§"+maxLength+"§"+minLength;
		   } 
  		   try {  
	  		   Process process = new ProcessBuilder(fpRunCommandChip.split("§")).start();
	  		   BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));   
	  		   String s;  
	  		   System.out.println(fpRunCommandChip.replace("§", " "));
	           Pattern pattern = Pattern.compile( ".*Reads used:\\s(\\d+).*");              

	  		   while ((s = results.readLine())!=null) {
	  			   System.out.println(s);	  			   
	  			   Matcher matcher = pattern.matcher( s );					
				   if (matcher.find()) {								
					   readNumberChip += Integer.parseInt(matcher.group(1));
				   }	 			   
	  		   }	 	  	   
			   progress = 30; setProgress(30);  
			   process = new ProcessBuilder(fpRunCommandControl.split("§")).start();
	  		   results = new BufferedReader (new InputStreamReader(process.getInputStream()));   
	  		   System.out.println(fpRunCommandControl.replace("§", " "));
	  		   while ((s = results.readLine())!=null) {
	  			   System.out.println(s);
	  			   Matcher matcher = pattern.matcher( s );					
				   if (matcher.find()) {								
					   readNumberControl += Integer.parseInt(matcher.group(1));
				   }	
	  		   }	 	  	   
			   progress = 33; setProgress(33);  
			   
			   if (genomeBlackList != null) {				 
				   DeleteRegions.runDeleteRegions(genomeBlackList, new File(outputDir,chipOutFilename));
				   progress = 36; setProgress(36); 
				   DeleteRegions.runDeleteRegions(genomeBlackList, new File(outputDir,controlOutFilename));
			   }
			   progress = 40; setProgress(40); 
			   Summary.createSummary (readNumberChip*1./readNumberControl,outputDir, chipOutFilename, controlOutFilename, summaryFilename);
			   progress = 50; setProgress(50); 

			   if (ratioForFiltering == null)
				   ratioForFiltering = 0f;
			   FilterPeaks.runFilterPeaks(new File(outputDir,chipOutFilename),new File(outputDir,controlOutFilename),ratioForFiltering,minimalCoverage);			   
			   progress = 70; setProgress(70); 
			   if (fpNumber != null){
				   MicsaRun(fpNumber);
				   new TextFrame("Motifs",motifs);
				   new TextFrame("Peaks Selected", new File(outputDir,micsaOutputFilename));
				   
			   }
			   progress = 100; setProgress(100); 
			   this.done(); 

  		   }catch (IOException e) {
				e.printStackTrace();
  		   }             
           return null;
        }
    	private void MicsaRun(Float fpNumber) {
    		
    		try{
    			
    			checkIfMEMEexists();
    			
    			String micsaCommand = "java§-Xmx1560m§-jar§micsa.jar§-name§"+micsaOutputFilename+"§-f§"+outputDir.getCanonicalPath()+File.separator+chipOutFilename;
    			micsaCommand += "§-n§"+fpNumber+"§-o§"+outputDir.getCanonicalPath()+"§-l§"+outputDir.getCanonicalPath()+File.separator+summaryFilename+"§-g§"+genomeDir.getCanonicalPath();
    			micsaCommand += "§-w§"+outputDir.getCanonicalPath()+"/chip_triangle_standard.wig.gz";
    		 
  	  		   	Process process = new ProcessBuilder(micsaCommand.split("§")).start();
  	  		   	BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));   
  	  		   	String s;  
  	  		   	System.out.println(micsaCommand.replace("§", " "));
  	  		   	Pattern patternNewMotif = Pattern.compile( ".*MEME found a new motif.*");              
  	  		   	Pattern patternNewSite = Pattern.compile( ".*site\\s\\d+.([ACGT]+)\\s(\\S).*");   

  	  		   	int motifCount = -1;
  	  		   	motifs = new ArrayList<ArrayList<String>>();
  	  //	site "+countSites+":"+ instance +" "+pVal
  	  		   	while ((s = results.readLine())!=null) {
  	  			   System.out.println(s);	  			   
  	  			   Matcher matcher = patternNewMotif.matcher( s );					
  				   if (matcher.find()) {								
  	  	  			   System.out.println(s);	  	  	  			   
  	  	  			   motifs.add(new ArrayList<String>());
  	  	  			   motifCount++;
  	  	  			   this.setProgress(this.getProgress()+1); 
  	  	  			   
  				   }	 
  				   matcher = patternNewSite.matcher( s );					
				   if (matcher.find()) {								
	  	  			   System.out.println(s);
	  	  			   motifs.get(motifCount).add(matcher.group(1));
				   }	
				   if (s.contains("Reading file with peaks")) {
  	  	  			   this.setProgress(this.getProgress()+2); 
				   }
				   if (s.contains("Reading wig file")) {
  	  	  			   this.setProgress(this.getProgress()+2); 
				   }
				   if (s.contains("wig file read")) {
  	  	  			   this.setProgress(this.getProgress()+4); 
				   }
				   if (s.contains("Genomic sequences of candidate peaks are set")) {
  	  	  			   this.setProgress(this.getProgress()+3); 
				   }
				   if (s.contains("This can take some time. Please wait")) {
  	  	  			   this.setProgress(this.getProgress()+5); 
				   }
				   if (s.contains("Running MEME to find other motifs")) {
  	  	  			   this.setProgress(this.getProgress()+2); 
				   }
				   if (s.contains("Running optimization procedure")) {
  	  	  			   this.setProgress(this.getProgress()+2); 
				   }
				   if (s.contains("Number of chromosomes read")) {
  	  	  			   this.setProgress(this.getProgress()+1); 
				   }
  	  		   	}	 	  	   
    		}catch (IOException e) {
 				e.printStackTrace();
   		   	}   
		}
	
		

		
		
		private void checkIfMEMEexists() {
			String OS = System.getProperty("os.name");
			String command;
			
			if (OS.contains("Windows")) {
				command = "meme.bin.exe";
			}
			else {
				command = "meme.bin";				
			}
			ProcessBuilder pb = new ProcessBuilder(command);
			pb.redirectErrorStream( true );
			Process process;
			try {
				process = pb.start();				
	
				BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));
				String s;			
				boolean MEMEfound = false; 
				while ((s = results.readLine())!=null) {				
					if (s.contains("USAGE")) {	
						MEMEfound = true;			
					}		
				}
				if (!MEMEfound) {
					this.setProgress(-1);
					this.cancel(true);				
				} 
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
			
			
		}
		@Override
        public void done() {
            Toolkit.getDefaultToolkit().beep();
            startButton.setEnabled(true);
            progressMonitor.setProgress(0);
        }
    }

    public ProgressMonitorMicsa(String aligner, File chipFile, File controlFile, File genomeDir, File outputDir, 
    		File genomeBlackList, Integer minLength, Integer maxLength, Integer meanLength, Integer maqQT, Float ratioForFiltering, Integer minimalCoverage, Float fpNumber) {
        super(new BorderLayout());

        this.aligner = aligner;
        this.chipFile = chipFile; 
        this.controlFile = controlFile;
        this.genomeDir= genomeDir;
        this.outputDir = outputDir; 
        this.minLength = minLength;
        this.maxLength= maxLength; 
        this.meanLength = meanLength;
        this.maqQT = maqQT;
        this.genomeBlackList = genomeBlackList;
        this.ratioForFiltering = ratioForFiltering;
        this.minimalCoverage = minimalCoverage;
        this.fpNumber = fpNumber;
        //Create the demo's UI.
        startButton = new JButton("Start");
        startButton.setActionCommand("start");
        startButton.addActionListener(this);

        taskOutput = new JTextArea(5, 50);
        taskOutput.setMargin(new Insets(5,5,5,5));
        taskOutput.setEditable(false);

        add(startButton, BorderLayout.PAGE_START);
        add(new JScrollPane(taskOutput), BorderLayout.CENTER);
        setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));


  	 
        
    }


    /**
     * Invoked when the user presses the start button.
     */
    public void actionPerformed(ActionEvent evt) {
        progressMonitor = new ProgressMonitor(ProgressMonitorMicsa.this,
                                  "Running the MICSA pipeline",
                                  "", 0, 100);
        progressMonitor.setProgress(0);
        fPrunTask = new FPrunTask();
        fPrunTask.addPropertyChangeListener(this);
        fPrunTask.execute();
        startButton.setEnabled(false);
    }

    /**
     * Invoked when task's progress property changes.
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ("progress" == evt.getPropertyName() ) {
            int progress = (Integer) evt.getNewValue();
            progressMonitor.setProgress(progress);
            String message =
                String.format("Completed %d%%.\n", progress);            
            progressMonitor.setNote(message);
            taskOutput.append(message);
            if (progress==-1){ //this means MEME cannot be found
    			String OS = System.getProperty("os.name");
    			message = "";
				if (OS.contains("Windows")) {					
					message += "Cannot run meme.bin.exe! Check its presence\n";
					message += "If meme.bin.exe is not in the directory your run MICSA from, please add MEME_DIRECTORY to your environment!";					
				}
				else {
					message += "Cannot run meme.bin! Check its presence and add MEME_DIRECTORY to your environment.\n";					
					message += "Try:\n";						
					message += "\t\t export MEME_DIRECTORY=/whereYouInstalledIt/meme_YourVersion/src";		
				}
	            taskOutput.append(message);
	            System.out.println(message);
	            System.out.println("Run of MICSA was canceled");
            }
            if (progressMonitor.isCanceled() || fPrunTask.isDone()) {
                Toolkit.getDefaultToolkit().beep();
                if (progressMonitor.isCanceled()) {
                	fPrunTask.cancel(true);
                    taskOutput.append("MICSA run was canceled.\n");
                } else {
                    taskOutput.append("MICSA run completed.\n");
                }
                //startButton.setEnabled(true);
                startButton.setVisible(false);
            }
        }
        /*
        if ("progress" == evt.getPropertyName() ) {
            int progress = (Integer) evt.getNewValue();
            progressMonitor.setProgress(progress);
            String message =
                String.format("Completed %d%%.\n", progress);
            progressMonitor.setNote(message);
            taskOutput.append(message);
            if (progressMonitor.isCanceled() || task.isDone()) {
                Toolkit.getDefaultToolkit().beep();
                if (progressMonitor.isCanceled()) {
                	fPrunTask.cancel(true);
                    taskOutput.append("Task canceled.\n");
                } else {
                    taskOutput.append("Task completed.\n");
                }
                startButton.setEnabled(true);
            }
        }
        */

    }

   
}
