package components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.krysalis.jcharts.chartData.ChartDataException;

import charts.BarCharts;

public class TextFrame extends JFrame{
    private JTextArea output;
    private ImagePanel imgPanel;
    static private String STARS = "******************************************";
	public TextFrame(String title, ArrayList<ArrayList<String>> motifs) {
		 this.setTitle(title);
		 output = new JTextArea(15, 50);
		 output.setMargin(new Insets(5,5,5,5));
		 output.setEditable(false);	        
	     add(new JScrollPane(output), BorderLayout.CENTER);

	     output.append("MOTIFS (this information was saved in motifs.txt):\n\n"+STARS+"\n");
	     
	     for (int i = 0; i<motifs.size(); i++) {
	    	 int mcount = i+1;
		     output.append("\nMOTIF "+mcount+"\n\n");
		     for (int j = 0; j <motifs.get(i).size(); j++) {
			     output.append(motifs.get(i).get(j)+"\n");
		     }
	     }
	     
		 this.pack();
	     Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	     this.setLocation(screenSize.width/3 - this.getSize().width/2, screenSize.height/3 - this.getSize().height/2);
	     this.show();
	}
	public TextFrame(String title, File peakFile) {
		 this.setTitle(title);
		 output = new JTextArea(15, 50);
		 output.setMargin(new Insets(5,5,5,5));
		 output.setEditable(false);	        
	     add(new JScrollPane(output), BorderLayout.CENTER);
	     output.append("SELECTED PEAKS (this information was saved in "+peakFile.getName()+"):\n\n"+STARS+"\n");
	     output.append("Chromosome\tStart\tEnd\tCoverageScore\tPeakLength\tP-value\n");

	     ArrayList <Integer> hist = new ArrayList <Integer>();
	     BufferedReader br = null;
     	 String s;
     	 //read peakFile
         try {              
           // Here BufferedInputStream is added for fast reading.
           br = new BufferedReader(new FileReader(peakFile));
           //chr14	105167617	105168778	78.558	1162	7.620773E-5
           Pattern pattern = Pattern.compile( ".*chr\\S+\\s\\d+\\s\\d+\\s(\\S+)\\s\\S+\\s\\S+.*");              
           while ((s = br.readLine())!=null) {	  			   
	  			   Matcher matcher = pattern.matcher( s );					
	  			   if (matcher.find()) {	
	  				   output.append(s+"\n");
						//String chrom = matcher.group(1);
						//Integer start = Integer.parseInt(matcher.group(2));
						//Integer end = Integer.parseInt(matcher.group(3));
	  				   Integer index = Math.round(Float.parseFloat(matcher.group(1)));
	  				   for (int i = hist.size(); i<= index; i++) {
	  					 hist.add(i, 0);
	  				   }
	  				   hist.set(index, hist.get(index)+1); 
					}	  			   
	  		  }	
           br.close();    
         } catch (IOException e) {
           e.printStackTrace();System.out.println(e.getMessage());
         }
	     
	     output.append(STARS+"\n");     
	     
	     String [] xLabeles = new String [28];
	     for (int i = 0; i<28; i++) {
	    	 xLabeles[i] = (new Integer(i+3)).toString();
	     }
	     
	     BarCharts barCharts= new BarCharts();
	      double[][] data= new double[1][28];
	      for (int i = 0; i<28; i++) {
	    	  data[0][i] = hist.get(i+3);
	      }
			String[] legendLabels= { "Selected peaks" };
		try {
			String graphFilename = (new File(peakFile.getParentFile(),"peakDistr")).getCanonicalPath();	
			barCharts.run("Peak depth distribution in MICSA output", xLabeles,"Peak depth", "Number of selected peaks",data,legendLabels,graphFilename);
			imgPanel = new ImagePanel( (new File(peakFile.getParentFile(),"peakDistr.png")).getCanonicalPath() );			
			//add panel to pane
			//this.add(imgPanel);	
			JFrame f = new JFrame();
		    f.getContentPane().setLayout(new BorderLayout());
		    JScrollPane ip = new JScrollPane(imgPanel);
		    f.add(ip, BorderLayout.CENTER);
		    f.setPreferredSize(new Dimension(500, 300));

		    f.setTitle("Peak Depth Distribution");
		    f.pack();
		    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		    f.setLocation(screenSize.width/4 - f.getSize().width/2, screenSize.height/4 - f.getSize().height/2);
		    f.show();
			
		} catch (ChartDataException e) {
				e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	     
	     
		 this.pack();
	     Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	     this.setLocation(screenSize.width/3*2 - this.getSize().width/2, screenSize.height/3*2 - this.getSize().height/2);
	     this.show();
	}
	
}
