package genome;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import PeakProcessing.Output;
import PeakProcessing.Peak;
import PeakProcessing.PeakClass;


import ru.genetika.common.ByteSequence;
import ru.genetika.common.CharSequence;
import ru.genetika.common.FastaByteSequenceReader;
import ru.genetika.common.FastaCharSequenceReader;
import ru.genetika.pwm.Pwm;
import ru.genetika.pwm.scanner.PwmScannerAhoC;
import ru.genetika.pwm.scanner.PwmScannerNaive;

import meme.Motif;

public class Genome {
	List <String> chrNames;
	List <ByteSequence> chrSeqs = null;
	int numberOfChromosomes = 0;
	Organism organism = null;
	Collection <String> chrFiles;
	File genomeDirectory;
	//List <StringBuffer> chrSeqsOLD;
	
	public Genome() {
		chrNames = new ArrayList<String>();
		chrSeqs = new ArrayList<ByteSequence>(); 
		numberOfChromosomes = 0;
		chrFiles = new ArrayList <String>();		
	}
	public String getFilename(String chromName) {
		for (String fileName : chrFiles) {
			String name = fileName.replaceAll(".fa.*", "");
			if (chromName.compareTo(name)==0) {
				return fileName;
			}
		}
		return null;
	}
	/*
	public Genome (Organism org) throws FileNotFoundException {
		this();		
		
		if (organism.getName().compareTo("human") == 0)
			this.chrFiles = organism.getChrFileNames();
		else {
			System.out.println("Unknown organism");
		}
		Iterator <String> it = this.chrFiles.iterator();	
		while(it.hasNext()) {
			String chrFileName = it.next();		
			this.addChomosomes(chrFileName);		
		}	
		
	}
	*/
	/*
	public Genome setGenome(Organism org) throws FileNotFoundException {
		Genome genome = new Genome();
		
		if (organism.getName().compareTo("human") == 0)
			chrFiles = organism.getChrFileNames();
		else {
			System.out.println("Unknown organism");
		}
		Iterator <String> it = chrFiles.iterator();	
		while(it.hasNext()) {
			String chrFileName = it.next();		
			genome.addChomosomes(chrFileName);		
		}	
		return genome;
	}
	*/
	/* public Genome(BufferedReader br) {
		chrNames = new ArrayList<String>();
		chrSeqs = new ArrayList<StringBuffer>();		
		
		
		try {
			while (br.ready()) {
				String s = br.readLine();
				
				if (s.matches(">.*")) {
					chrNames.add(new String(s));
					chrSeqs.add(new StringBuffer(""));		
					numberOfChromosomes++;
			    }
			    else {
			    	//String st = s.toString().replaceAll("\n", "");
			    	//chrSeqs.get(numberOfChromosomes-1).append(st);
			    	//System.out.println(chrSeqs.get(0).length());
			    	chrSeqs.get(numberOfChromosomes-1).append(s);
			    }				
			}			
		}
		catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
		}
	}
	*/
	
	/*
	public Genome(String fileName) throws FileNotFoundException {
		this();
		this.addChomosomes(fileName);	
	}		
	*/
	/*
	public Genome(File file) throws FileNotFoundException {
		this();
		this.addChomosomes(fileName)
		
		FastaByteSequenceReader reader = new FastaByteSequenceReader(file);
		
		ByteSequence seqB = null;
			
		try {
			seqB = reader.getSequence();			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			
			
		chrNames.add(new String(seqB.getName()));
		chrSeqs.add(seqB);		
		numberOfChromosomes++;		
	}
	*/
	
	
	public Genome(File genomeDirectory) {
		this();
		this.genomeDirectory = genomeDirectory;
		String[] files = null;
		if (genomeDirectory.isDirectory()) {
		      files = genomeDirectory.list();
		      if (files.length == 0) 
		        throw new IllegalArgumentException(
		            "Directory is empty: " + genomeDirectory);		      
		}
		for (String fileName : files) {
			if (fileName.contains(".fa")||fileName.contains("chr")) {
				chrFiles.add(fileName);
				String name = fileName.replaceAll(".fa.*", "");
				chrNames.add(name);
				numberOfChromosomes++;
				//this.addChomosomes(new File (genomeDirectory,fileName));				
				//System.out.println(name);
			}			
		}		
	}

/*
	public static Genome setGenome(String fileName) throws FileNotFoundException {
		Genome genome = new Genome(); 
		
		BufferedReader br_all = new BufferedReader (new FileReader (fileName));
		
		try {	
			while (br_all.ready()) {
				String s = br_all.readLine();
				System.out.println("Reading file "+s);
				genome.addChomosomes(s);								
			}			
		}
		catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
		}				
		return genome;
	}
	
	private void addChomosomes(File file) {
		
		FastaByteSequenceReader reader = new FastaByteSequenceReader(file);
		ByteSequence seqB = null;
		try {
			seqB = reader.getSequence();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			
		
		chrNames.add(new String(seqB.getName()));
		chrSeqs.add(seqB);		
		numberOfChromosomes++;
		
	}

	
	private void addChomosomes(String fileName) throws FileNotFoundException {
		
		FastaByteSequenceReader reader = new FastaByteSequenceReader(new File(fileName));
		ByteSequence seqB = null;
		try {
			seqB = reader.getSequence();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			
		
		chrNames.add(new String(seqB.getName()));
		chrSeqs.add(seqB);		
		numberOfChromosomes++;
	}
*/
	public int getNumberOfChromosomes () {
		return numberOfChromosomes;
	}
	
	public void setPeakSequences(Peak peak) {
		String chr = peak.getChromosome(); 
		ByteSequence bs = getByteStringFromChrom(chr);
		int start = peak.getPeakStart();
		int length = peak.getSequenceLength();
		
		String SubString = bs.getSubString(start, length);
		peak.setSequence(SubString);
	}
	
	private ByteSequence getByteStringFromChrom (String chromosome) {
		Iterator <String> it = chrNames.iterator();
		int number = 0;
		while (it.hasNext()) {
			if (chromosome.compareToIgnoreCase(it.next())==0) {
				break; 
			}
			number ++;
		}
		if (number == chrNames.size())
			return null;
		return chrSeqs.get(number);
	}
	
public void setMotifProbabilities(Motif motif) {
		
		System.out.println("Seting motif probabilities");
		
		String chromosome = chrNames.get(0);
		String filename = getFilename(chromosome);
		File file = new File (genomeDirectory, filename);
		
		FastaCharSequenceReader reader = new FastaCharSequenceReader(file);
		
		//ByteSequence seqB = null;
		CharSequence seqB = null;
		try {
			seqB = reader.getSequence();			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}			
		
		int totalTextLength = 0;
		
		/*	
		try {
			seqB = reader.getSequence();			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}			
		
		int totalTextLength = 0;
		
		/*
		PwmScannerAhoC scanner = new PwmScannerAhoC();
		scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));		
					
		scanner.setSequence(seqB);
			
			
		scanner.setMinThreshold(motif.getMinThreshold());
			
		scanner.scanToProb(motif);					
		totalTextLength += seqB.getlLengthNotN();			
			
		System.gc();
		motif.recalcProbs(totalTextLength);
		motif.recalcThresholds(); //delete unused thresholds		
		*/
		PwmScannerNaive scanner = new PwmScannerNaive();
		scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));		
		
		scanner.setSequence(seqB);
		scanner.setMinThreshold(motif.getMinThreshold());
		totalTextLength += scanner.scanToProb(motif);				
			
		System.gc();
		motif.recalcProbs(totalTextLength);
		motif.recalcThresholds(); //delete unused thresholds		

}
	
	

	/*
	public void setMotifProbabilities(Motif motif) {
		
		System.out.println("Seting motif probabilities");
		int totalTextLength = 0;
		
		
		PwmScannerAhoC scanner = new PwmScannerAhoC();
		scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));		
		
		Iterator <ByteSequence> it = chrSeqs.iterator(); 
				
		while (it.hasNext()) {
			ByteSequence seqB = it.next();					
				
			scanner.setSequence(seqB);
			
			
			scanner.setMinThreshold(motif.getMinThreshold());
			//scanner.setResultListener(new ResultPrinter());
			//scanner.scan();
			scanner.scanToProb(motif);					
			totalTextLength += seqB.getlLengthNotN();			
		}	
		System.gc();
		motif.recalcProbs(totalTextLength);
		motif.recalcThresholds(); //delete unused thresholds
		
		*/
		/*
		PwmScannerNaive scanner = new PwmScannerNaive();
		scanner.setPwm(new Pwm(motif.getPSSM(),motif.getName()));		
			
		CharSequence seqC = null;
		
		FastaCharSequenceReader reader = new FastaCharSequenceReader(new File("chr22.fa"));
		try {
			seqC = reader.getSequence();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		scanner.setSequence(seqC);
		scanner.setMinThreshold(motif.getMinThreshold());	
				
		totalTextLength += scanner.scanToProb(motif);			
			
		motif.recalcProbs(totalTextLength);
		
		*/
		/*}
	
	*/
	public double [] getLetterFreqs() {
		double [] freq = new double [4];
		String chromosome = chrNames.get(0);
		String filename = getFilename(chromosome);
		File file = new File (genomeDirectory, filename);
		LineNumberReader reader;
		int lineNumber = 1000000; //mirovaya konstanta 
		StringBuffer sb = null;
		try {
			reader = new LineNumberReader (new FileReader(file));
		
			sb = new StringBuffer();
		
			if (reader.ready()) {
				reader.readLine(); //must be fasta format				
			}
			while(reader.ready()&&(lineNumber>0))	{			
				sb.append(reader.readLine().toUpperCase());
				lineNumber--;
			}
			reader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}		
		float count_a = 0;
		float count_c = 0;
		float count_g = 0;
		float count_t = 0;

		for (int i = 0; i < sb.length(); i++){
			
			if ( sb.charAt(i) == 'A')
				count_a++;	
			else if ( sb.charAt(i) == 'C')	
				count_c++;
			else if ( sb.charAt(i) == 'G')	
				count_g++;
			else if ( sb.charAt(i) == 'T')	
				count_t++;
		}

		float total_count = count_a +count_c + count_g + count_t;
		freq[0] = count_a/total_count;
		freq[1] = count_c/total_count;
		freq[2] = count_g/total_count;
		freq[3] = count_t/total_count;
		
		return freq;
	}
	public void setPeakSequences(Output myOutput) throws IOException {
		StringBuffer sb; 

		for (String chromosome : chrNames) {
			String filename = getFilename(chromosome);
			File file = new File (genomeDirectory, filename);
			System.out.println("Setting peak sequences for "+chromosome);
			LineNumberReader reader = new LineNumberReader (new FileReader(file));
			//byte[] seq = new byte[(int) file.length()];
			//Arrays.fill(seq, (byte)'\0');
			//int pos = 0;
			//boolean ifFirstSequenceRead = false; 
				
			myOutput.setTruePeakSequencesAreSet(chromosome);
			
			if (reader.ready()) {
				String s = reader.readLine(); //must be fasta format
				if (!s.contains(">")) {
					System.err.println(filename + " must be in FASTA format");
					System.exit(1);
				}
			}
			String s = reader.readLine(); //to know length of line
			int lineLength = s.length();
		//	int correction = reader.getLineNumber();

						
			for (int i : myOutput.getValuesOfClasses()) {
				PeakClass pk = myOutput.getPeakClass(chromosome, i);
						
				if (pk != null) {
					reader.close();
					reader = new LineNumberReader (new FileReader(file));
					reader.readLine(); //read the first line with a name
					int totalSymbolsRead = 0;
				//	int firstLineLength = 0;
					
					for (Peak peak : pk.getAllPeaks()) {
						//counter++;
						int ci = 0;
						int n = (int)'\n';
						int start = peak.getPeakStart();
						int end = peak.getPeakEnd();
						//System.out.println(counter);//+" Setting peak sequences for "+start + " "+ end);
						//System.out.println(".");
						while (totalSymbolsRead<start) {
							ci = reader.read();
							if (ci != n) {
								totalSymbolsRead++;
							}
							else {
								int linesLeft = (start - totalSymbolsRead-1)/lineLength;
								reader.skip(linesLeft*(lineLength+1));
								totalSymbolsRead+= linesLeft*lineLength;
								//System.out.println("line read");
							}
						}
						sb = new StringBuffer();
						sb.append((char)ci);
						
						for (int j = 0; j<end-start;j++) {
							char c = (char)reader.read();
							if (c == '\n') 
								j--;
							else {
								sb.append(c);
								totalSymbolsRead++;
							}
						}				
						peak.setSequence(sb.toString().toUpperCase());
						//if (start == 554014) {
						//	System.out.println(sb.toString().toUpperCase());
						//	System.out.println(peak.getCoverage());
						//}
					}				
					
					
					
				}
				//System.out.println(i+"");
				//int counter = 1;
									
			}	
			reader.close();
					
			
			
			
			
			/*
			for (int i : myOutput.getValuesOfClasses()) {
				PeakClass pk = myOutput.getPeakClass(chromosome, i);
								
				System.out.println("Setting peak sequences for class "+i);
				for (Peak peak : pk.getAllPeaks()) {					
					int start = peak.getPeakStart();
					int end = peak.getPeakEnd();
					System.out.println("Setting peak sequences for "+start + " "+ end);

					int lineToRead = (start-1)/lineLength+2;
					int linesToSkip = lineToRead-correction;
					int symbolsToSkip = start+firstLineLength+linesToSkip;
					//reader.reset();
					reader.close();
					reader = new LineNumberReader (new FileReader(file));
					reader.skip(symbolsToSkip);	
					
					StringBuffer sb = new StringBuffer();
					
					int peakLengthInLines = (end-start)/lineLength+2;
					for (int j  = 0; j<peakLengthInLines; j++) {
						sb.append(reader.readLine());							
					}					
					String SubString = sb.substring(0, end-start+1).toUpperCase();	 //genome coordinates start by 1, not by 0
					peak.setSequence(SubString);
					totalSymbolsRead = start +
				}									
			}
			
			
			reader.close();
			*/
			
			/*
			 
			sb = new StringBuffer();

			if (reader.ready()) {
				String s = reader.readLine(); //must be fasta format
				if (!s.contains(">")) {
					System.err.println(filename + " must be in FASTA format");
					System.exit(1);
				}
			}
			while(reader.ready())
			{			
				sb.append(reader.readLine());				
			}
			reader.close();
			
			for (int i : myOutput.getValuesOfClasses()) {
				PeakClass pk = myOutput.getPeakClass(chromosome, i);
				for (Peak peak : pk.getAllPeaks()) {					
					int start = peak.getPeakStart();
					int end = peak.getPeakEnd();	
					//if (end>=sb.length()) {
					//	pk.remove(peak);
					//}
					//else {
						String SubString = sb.substring(start-1, end).toUpperCase();	 //genome coordinates start by 1, not by 0
						if (start == 554014) {
							System.out.println(SubString);
							for (float x:peak.getCoverage())
							{
								System.out.println(x);
							}
						}
						peak.setSequence(SubString);
				//	}
							
				}
			}
			
			//System.gc();
			*/
		}
		//check
		List <String> ChrNameList = new ArrayList<String>(myOutput.getAllChromosomes()); 
		for (String chromosome: ChrNameList) {
			if(myOutput.isPeakSequencesAreSet(chromosome)==false) {
				myOutput.removePeaksAtChromosome(chromosome);
			}			
		}
	}
	
}
