package meme;

import genome.Genome;

import java.io.*;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ru.genetika.common.*;
import ru.genetika.common.CharSequence;

import ru.genetika.pwm.Pwm;
import ru.genetika.pwm.scanner.PwmScannerNaive;

public class Meme {	
	   
		boolean err = false;
		String nameOfFileWithSeqs;  // file with sequences to run MEME on
		String nameOfSecondFileWithSeqs = "tmp.txt";  // the second file with sequences to run MEME on, doesn't contain sequences containing the first identified motif
		List <Motif> myMotifs;		// List of motifs identified by MEME
		int NumberOfMotifs;			// Number of motifs identified by MEME
		File outputdirectory;       // Output directory
		
		public Meme (File outputdirectory, String nameOfFileWithFastaSeqs) {
			nameOfFileWithSeqs = nameOfFileWithFastaSeqs; 
			myMotifs = new ArrayList <Motif> ();
			this.outputdirectory = outputdirectory;
			try		{
				System.out.println("Running MEME\nThis can take some time. Please wait.\n");
				String OS = System.getProperty("os.name");
				String command;
				if (OS.contains("Windows")) {
					command = "meme.bin.exe§"+outputdirectory+"\\"+nameOfFileWithSeqs+"§-revcomp§-dna§-mod§zoops§-nmotifs§1§-evt§0.1§-minw§7§-maxw§24§-text§-maxsize§700000§-nostatus";
				}
				else {
					command = "meme.bin§"+outputdirectory+"/"+nameOfFileWithSeqs+"§-revcomp§-dna§-mod§zoops§-nmotifs§1§-evt§0.1§-minw§7§-maxw§24§-text§-maxsize§700000§-nostatus";
					//System.out.println(command);
					String MEME_Directory = new ProcessBuilder().environment().get("MEME_DIRECTORY");
					if (MEME_Directory==null) {
						System.out.println("Please add MEME_DIRECTORY to your environment!");
						System.out.println("Try:");						
						System.out.println("\t\t export MEME_DIRECTORY=/whereYouInstalledIt/meme_YourVersion/src");						

					}
				}
				
				Process process = new ProcessBuilder(command.split("§")).start();

				BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));
				String s;
				int countSites = 0;
				//   chrX:36408574-36409167       -    102  3.21e-15 AAGGAGGGAG GGAAGGAAGGAAGGAAGGAAGGAA AAATAAATCT
				Pattern pattern = Pattern.compile( "\\.*\\:\\d+\\-\\d+\\s+[+-]\\s+\\d+\\s+([0-9\\.e-]+)\\s+\\S*\\s(\\S+)\\s.*");

				while ((s = results.readLine())!=null) {				
					if (s.matches("MOTIF.*")) {	
						System.out.println("MEME found a new motif\n"+s);
						myMotifs.add(new Motif(s));
						countSites = 0;					
					}				
					
					if (s.matches("Stopped.*")) {	
							NumberOfMotifs = myMotifs.size(); 
							System.out.println("MEME "+ s);
							break;
					}
											
					Matcher matcher = pattern.matcher( s );					
					if (matcher.find()) {								
						Float pVal = Float.parseFloat(matcher.group(1));
						String instance = matcher.group(2);						
						System.out.println("site "+countSites+":"+ instance +" "+pVal);
						myMotifs.get(myMotifs.size()-1).setSite(countSites,instance);	
						countSites++;		
					}
							
						
				}
				BufferedReader errors = new BufferedReader( new InputStreamReader(process.getErrorStream()));
				
				while ((s = errors.readLine())!= null) {
					System.out.println(s);
					err = true;
				}
		 
			
			 //some more code
			 
			}catch(Exception e){
				throw new RuntimeException(e);
				}
			if(err)
				throw new RuntimeException ("Errors executing MEME");
			System.out.println("First round of MEME is done");					
			
		}
		public int getNumberOfMotifs () {
			return NumberOfMotifs;
		}
		public float [][] getPSSM (int i) {
			if (i >= NumberOfMotifs) {
				return null;
			}
			return myMotifs.get(i).getPSSM(); 
		}
		public String getName(int i) {
					
			if (i >= NumberOfMotifs) {
				return null;
			}
			return myMotifs.get(i).getName();			
		}
		public Float getMinThreshold(int i) {			
			return (Float) myMotifs.get(i).getMin_thresh();
		}
		public void fillPSSM(double[] letterFreqs) {
			for (int i = 0; i< NumberOfMotifs; i++)
				myMotifs.get(i).fillPSSM(letterFreqs);
			
		}
		public void fillThresholds() {
			for (int i = 0; i< NumberOfMotifs; i++)
				myMotifs.get(i).setMaxMinThresholds();
			
		}
		public Motif getMotif(int i) {			
			return myMotifs.get(i);
		}
		public int secondRun(Genome genome, double[] letterFreqs) {
			if(!checkIfOneMotif ()) {
				discardSecondMotif();
			}
			try {
					List <String> filesToDelete = new ArrayList <String>();
					int maxNumberOfAdditionalRounds = 2;
				for (int round = 0; round <maxNumberOfAdditionalRounds;round++) {
										
					Integer randD = ((Double)(Math.random()*1000)).intValue();
					String nameOfNewFileWithSeqs = randD.toString()+nameOfSecondFileWithSeqs;
					filesToDelete.add(nameOfNewFileWithSeqs);
					createFileWOsequencesContainingMotif(round,nameOfNewFileWithSeqs);
					System.out.println("Running MEME to find other motifs\n");
					String OS = System.getProperty("os.name");
					String command;
					if (OS.contains("Windows")) {
						command = "meme.bin.exe§"+outputdirectory+"\\"+nameOfNewFileWithSeqs+"§-revcomp§-dna§-mod§zoops§-nmotifs§1§-evt§0.5§-minw§7§-maxw§24§-text§-maxsize§500000§-nostatus";
					}
					else {
						command = "meme.bin§"+outputdirectory+"/"+nameOfNewFileWithSeqs+"§-revcomp§-dna§-mod§zoops§-nmotifs§1§-evt§0.5§-minw§7§-maxw§24§-text§-maxsize§500000§-nostatus";
					}
									
					Process process = new ProcessBuilder(command.split("§")).start();
					BufferedReader results = new BufferedReader (new InputStreamReader(process.getInputStream()));
					//   chrX:36408574-36409167       -    102  3.21e-15 AAGGAGGGAG GGAAGGAAGGAAGGAAGGAAGGAA AAATAAATCT
					Pattern pattern = Pattern.compile( "\\.*\\:\\d+\\-\\d+\\s+[+-]\\s+\\d+\\s+([0-9\\.e-]+)\\s+\\S*\\s(\\S+)\\s.*");
	
					String s;
					int countSites = 0;
					while ((s = results.readLine())!=null) {						
					    Matcher matcher = pattern.matcher( s );					
						if (matcher.find()) {								
							Float pVal = Float.parseFloat(matcher.group(1));
							String instance = matcher.group(2);						
							System.out.println("site "+countSites+":"+ instance +" "+pVal);
							myMotifs.get(myMotifs.size()-1).setSite(countSites,instance);	
							countSites++;		
						}
						
						if (s.matches("MOTIF .*")) {	
							System.out.println("MEME found a new motif\n"+s);
							myMotifs.add(new Motif(s));
							countSites = 0;					
						}				
						if (s.matches("Stopped.*")) {	
							NumberOfMotifs = myMotifs.size(); 
							System.out.println("MEME "+ s);
							if (countSites==0) {
								round = maxNumberOfAdditionalRounds;
							}
							break;
						}								
						
					}
					if (countSites>0) {
						int lastMotifIndex = myMotifs.size()-1;
						getMotif(lastMotifIndex).fillPSSM(letterFreqs);		
						getMotif(lastMotifIndex).setMaxMinThresholds();
						genome.setMotifProbabilities(getMotif(lastMotifIndex));						
					}
					
					BufferedReader errors = new BufferedReader( new InputStreamReader(process.getErrorStream()));
					
					while ((s = errors.readLine())!= null) {
						System.out.println(s);
						err = true;
					}	
				}
				for (String fileNameToDelete : filesToDelete) {
					File fileToDelete = new File(outputdirectory,fileNameToDelete);
					if (!fileToDelete.exists())
					      throw new IllegalArgumentException(
					          "Delete: no such file or directory: " + fileToDelete);

					if (!fileToDelete.canWrite())
					      throw new IllegalArgumentException("Delete: write protected: "
					          + fileToDelete);

					    // If it is a directory, make sure it is empty
					if (fileToDelete.isDirectory()) {
					      String[] files = fileToDelete.list();
					      if (files.length > 0)
					        throw new IllegalArgumentException(
					            "Delete: directory not empty: " + fileToDelete);
					}

					    // Attempt to delete it
					boolean success = fileToDelete.delete();

					if (!success)
					      throw new IllegalArgumentException("Delete: deletion failed");		
				}			
							
				if (myMotifs.size()>=2)   
					return myMotifs.size()-1;
				return 0;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
			return 0;
		}
		
		
		
		private void createFileWOsequencesContainingMotif(int motifNumber, String Filename) throws IOException {		
			File newFile = new File (outputdirectory,Filename);
			FileWriter fstream = new FileWriter(newFile);
	        BufferedWriter out = new BufferedWriter(fstream);
			
			FastaCharSequenceReader reader = new FastaCharSequenceReader(new File(outputdirectory,nameOfFileWithSeqs));		
			ListCharSequence seqsC = null;			
			try {
				seqsC = reader.getSeqList();
			} catch (IOException e) {
					// TODO Auto-generated catch block
				e.printStackTrace();
			}
			PwmScannerNaive scanner = new PwmScannerNaive();
			scanner.setPwm(new Pwm(getPSSM(motifNumber),getName(motifNumber)));							
			if ((myMotifs.get(motifNumber).probs!=null)&&( myMotifs.get(motifNumber).getMin_thresh() != null)&&( myMotifs.get(motifNumber).probs.get(myMotifs.get(motifNumber).getMin_thresh()) != null))
				scanner.setPwmProb(myMotifs.get(motifNumber).probs.get(myMotifs.get(motifNumber).getMin_thresh()));
			scanner.setMinThreshold(getMinThreshold(motifNumber));	
			
			for (int i = 0; i<seqsC.getSize();i++) {				
				CharSequence seqC = (CharSequence) seqsC.getSequence(i);				
				scanner.setSequence(seqC);					
				
				if(scanner.scanToSample().getNumberOfSites()==0) {
					//add this sequence and its name to the new file
					out.write(">"+seqsC.getName(i)+"\n");
					char [] nucleoSequence = (char[])seqsC.getSeqArr(i);
					for (int j = 0; j<nucleoSequence.length;j++)
						if (nucleoSequence[j] != '\0')
							out.write(nucleoSequence[j]);
					out.write("\n");
				}
				
			}
			//Close the output stream
			out.close();
			nameOfFileWithSeqs = Filename;
		}
		private void discardSecondMotif() {
			for (int i=NumberOfMotifs - 1; i>0; i--) {
				myMotifs.remove(i);
			}
			NumberOfMotifs = 1;				
		}
		private boolean checkIfOneMotif() {
			if (NumberOfMotifs == 1)
				return true;
			return false;			
		}
		public List<Motif> getMotifs() {
			
			return myMotifs;
		}
		
}

