package meme;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.lang.Math;


/* a known motif */
public class Motif {
		
	  private String name;		 		/* names of motif */
	  private int width;				/* (known) width of motif */
	  private float min_thresh;			/* minimum threshold for motif */	  
	  private float max_thresh;			/* maximum threshold for motif */
	  private String [] sites;			/* best nsites this motif */
	  private int number_sites;			/* number_of_sites */
	  //public float thresh;			/* best threshold this motif */
	  private float [][] PSSM;			/* PSSM for this motif */
	  private double pseudoScore ;			/* number of pseudoScores */
	  private double Evalue;			/* Total E-value */
	  private double Pvalue;					/* Total P-value */
	  
	  private static float precision = 100; //precision for the minimal threshold value : 100 means precision 0,01	   
	  private static int tStep = 1; /* step for threshold values */
	  
	  HashMap <Float,Float>  probs; /* probabilities for this motif for each threshold value*/
	  List <Float> thresholds;       /* list of threshold values*/
	  
	  public Motif (String s) {
		  Pattern pattern = Pattern.compile( "(MOTIF\\s+\\d+)\\s+width =\\s+(\\d+)\\s+sites =\\s+(\\d+).*E-value =\\s+([0-9\\.\\-e]+).*");

		  //MOTIF  1	width =   24   sites = 115   llr = 3793   E-value = 1.8e-1187
		  Matcher matcher = pattern.matcher( s );					
		  if (matcher.find()) {		
			  
			  name = matcher.group(1);
			  width = Integer.parseInt(matcher.group(2));
			  number_sites = Integer.parseInt(matcher.group(3));
			  Evalue = Double.parseDouble(matcher.group(4));
			  Pvalue = Double.parseDouble(matcher.group(4));
			  sites = new String [number_sites];
			  PSSM = new float [width][4];
			  pseudoScore = 1;			  
		  }
		  else {
			  System.exit(1);
		  }	  
		 
	  }
	  
	  public Float getMin_thresh () {
		  return min_thresh;
	  }
	  
	  public void fillPSSM (double [] freqs) {
		  double [][] counts = new double [width][4];
		  PSSM = new float [width][4];
		  pseudoScore = Math.log(number_sites);
		  
		  for( int i = 0; i < width; i++ ) {		      
		      for (int j = 0; j<4;j++) 
		    	  counts[i][j] = 0;
		      for (int m = 0; m<number_sites; m++) {
		    	    switch (sites[m].charAt(i)) {
	      			case 'a': counts[i][0]++; break;
	      			case 'A': counts[i][0]++; break;
	      			case 'c': counts[i][1]++; break;
	      			case 'C': counts[i][1]++; break;
	      			case 'g': counts[i][2]++; break;
	      			case 'G': counts[i][2]++; break;
	      			case 't': counts[i][3]++; break;
	      			case 'T': counts[i][3]++; break;	    	  
		    	   }
		      }
		      for (int j = 0; j<4;j++) 		
		    	  	PSSM[i][j] = (float) (precision*Math.log(( (counts[i][j] + pseudoScore * freqs[j]) / (number_sites + pseudoScore) ) / freqs[j]));	     
		  
		  }  
		  
	  }
	  /*
	  public void searchText (FastaText fText) {
		  searchText(fText,min_thresh);
	  }
	  public void searchText (FastaText fText, double threshold) {
		  //TODO
	  }
	  */
	  public void setMaxMinThresholds (float minTh, float maxTh) {
		  min_thresh = (float) Math.floor(minTh);		/* minimum threshold for motif */	  
		  max_thresh = maxTh + tStep;		/* maximum threshold for motif */
		  
		  probs = new HashMap <Float,Float>();
		  thresholds = new ArrayList <Float> () ;     
		  
		  for (float t = min_thresh; t<= max_thresh; t = t+tStep) {			 	
			  t = (float) Math.round(t);
			  probs.put(t, 0f);
			  thresholds.add(t);
			 // System.out.println(t);
		  }	  
	  }
	  public void setMaxMinThresholds () {
		  //find minTh, maxT
		  float minTh = 1000000;
		  float maxTh = 0;
		  
		  for (int i = 0; i < number_sites; i++) {
			  float Score = 0;
			  for (int j = 0; j < width; j++) {
				  Score += PSSM(j,sites[i].charAt(j));
			  }
			  if (minTh>Score){
				  minTh = Score;
			  }
			  if (maxTh<Score){
				  maxTh = Score;
			  }
		  }
		  maxTh = (float) Math.ceil(maxTh);	 
		  double toSub = Math.floor(4*precision*minTh/(maxTh-minTh));
		  if (minTh-(float)toSub<minTh/2) {
			  toSub = minTh/2;
		  }
		  if (toSub>500) { //mirovaya konstanta
			  toSub = 500;
		  }
		  if (minTh-(float)toSub<300) {
			  toSub = minTh-300;
		  }
		 // toSub = 0;
		  //System.out.println("Threshold will be loweredf from " +minTh+" by "+toSub);
		  //setMaxMinThresholds ( minTh-(float)toSub,  maxTh);	//100 ili 50 - mirovaya konstanta	  
		  setMaxMinThresholds ( minTh-(float)toSub,  maxTh);	//100 ili 50 - mirovaya konstanta	  
	  }


	private double PSSM(int i, char a) {		
		return PSSM[i][n(a)];
	}


	private int n(char a) {
		
		switch (a) {		
		
    	case 'a': return 0;
    	case 'A': return 0;
    	case 'c': return 1;
    	case 'C': return 1;
    	case 'g': return 2;
    	case 'G': return 2;
    	case 't': return 3;
    	case 'T': return 3;
		}
		return -1;
	}


	public float getMinThreshold() {
		
		return min_thresh;
	}


	public float [][] getPSSM() {
		
		return PSSM;
	}


	public String getName() {
		
		return name;
	}
	
	public List <Float> getThresholds () {
		return thresholds;
	}
	public float [] getThresholdsAsArray () {
		
		float [] thresh = new float [thresholds.size()];
		
		for (int i = 0; i< thresholds.size(); i++) {
			thresh[i] = thresholds.get(i);
		}
		
		return thresh;
	}


	public void addOcc(float thresh) {
		if (probs.containsKey(thresh)) {
			probs.put(thresh,probs.get(thresh)+1);		
		}
		else {
			probs.put(thresh,1f);
			thresholds.add(thresh);
			Collections.sort(thresholds); 

		}
		
	}
	  
	public void recalcProbs(int textLength) {
		
		for (int i = thresholds.size()-1; i>0; i--) {  //nothing to do for the first value
			float thresh1 = thresholds.get(i);
			float value = probs.get(thresh1);
			
			float thresh2 = thresholds.get(i-1);
			float value2 = probs.get(thresh2);
			probs.put(thresh2, value2+value);				
						
		}
		for (int i = 0; i<thresholds.size(); i++) {  
			float thresh1 = thresholds.get(i);
			float value = probs.get(thresh1);
			//float tmp = value/textLength;
			probs.put(thresh1, value/textLength);		
		}		
	}
	
	public double getProb (float f) {
		if (probs.containsKey(f))
			return probs.get(f);
		if (f<min_thresh)
			return 1f;
		if (f>max_thresh)
			if (probs.containsKey(max_thresh))
				return probs.get(max_thresh);
			else
				return probs.get(max_thresh-1);
		float closestKey = min_thresh;
		float dist = f-closestKey;
		for (float key:probs.keySet()) {
			if ((f-key>=0)&&(f-key<=dist)) {
				dist = f-key;
				closestKey = key;
			}
		}
		return probs.get(closestKey);
	}


	public float getPrecision() {		
		return precision;
	}

	public void setSite(int i, String s) {
		sites[i] = s;		
	}

	public void recalcThresholds() {
		//System.out.println(thresholds.size()); 
		//System.out.println(probs.size());

		float thresh1 = thresholds.get(1); //starting from 1 to not remove the minimal threshold
		float value1 = probs.get(thresh1);
		
		for (int i = 2; i<thresholds.size()-1; i++) {  
			float thresh2 = thresholds.get(i);
			float value2 = probs.get(thresh2);
			if (value1 == value2) {
				thresholds.remove(thresh1);
				probs.remove(thresh1);				
			}
			thresh1 = thresh2;
			value1 = value2;					
		}	
		//System.out.println(thresholds.size());
		//System.out.println(probs.size());

	}

	public int getWidth() {		
		return width;
	}
}
