package micsa.fun;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class FilterPeaks {
	public static void runFilterPeaks(File chipFile, File controlFile, Float ratioForFiltering, Integer minimalCoverage) {
		
		Hashtable <String,ArrayList<Integer>> contrlL = new Hashtable <String,ArrayList<Integer>>(); 
		Hashtable <String,ArrayList<Integer>> contrlR = new Hashtable <String,ArrayList<Integer>>();  
		Hashtable <String,ArrayList<Float>> contrlS = new Hashtable <String,ArrayList<Float>>(); 		
      	readToHash(controlFile,contrlL,contrlR,contrlS);
		
      	Pattern pattern = Pattern.compile( ".*(chr.*)\\s(\\d+)\\s(\\d+)\\s\\d+\\s([0-9\\.]+).*");              
		String s;
    	try {
    		BufferedReader br = new BufferedReader(new FileReader(chipFile));
            StringBuffer sb = new StringBuffer();  

			while ((s = br.readLine())!=null) {
				   //System.out.println(s); 
				   s = s.replace(".fa", "");
				   Matcher matcher = pattern.matcher( s );					
				   if (matcher.find()) {								
						String chrom = matcher.group(1);
						Integer left = Integer.parseInt(matcher.group(2));
						Integer right = Integer.parseInt(matcher.group(3));
						Float score = Float.parseFloat(matcher.group(4));
						boolean exclude = false;
						if (Math.round(score)<minimalCoverage) {
							exclude = true;
						}
						else if (ratioForFiltering>0){
							exclude = check (ratioForFiltering, left,right,score,contrlL.get(chrom),contrlR.get(chrom),contrlS.get(chrom));
						}
						if (!exclude)
							sb.append(s+"\n");
				   }
				   else {
					   sb.append(s+"\n");
				   }
			}
			br.close();
			
			BufferedWriter bw = new BufferedWriter(new FileWriter(chipFile));            
	        bw.write(sb.toString()) ;	
	        bw.close() ;     
			
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}			      	
	}
	private static boolean check(Float ratioForFiltering, Integer x, Integer y, Float score,
			ArrayList<Integer> arrL, ArrayList<Integer> arrR,
			ArrayList<Float> arrS) {
		
		for (int i = 0; i <arrL.size(); i++) {
			if (Math.abs(x-arrL.get(i))<100000){
				int dist = dist(x,y,arrL.get(i),arrR.get(i));
				//Float ratio = score/(arrS.get(i));
				if ((dist == 0)&&(score/(arrS.get(i))<=ratioForFiltering)) {						
					return true ;
				}
			}	
		}	
		return false;
	}
	private static int dist(Integer x, Integer y, Integer a, Integer b) {
		
		if (y<a) {
			return a-y;
		}
		if (b<x) {
			return x-b;
		}
		return 0;
	}
	private static void readToHash(File file, Hashtable<String, ArrayList<Integer>> arrL,
			Hashtable<String, ArrayList<Integer>> arrR,
			Hashtable<String, ArrayList<Float>> scor) {
   		Pattern pattern = Pattern.compile( ".*?(c?h?r?[0-9XYMTZW]+)\\s(\\d+)\\s(\\d+)\\s\\d+\\s([0-9]+\\.[0-9]*).*");              
		String s;
    	try {
    		BufferedReader br = new BufferedReader(new FileReader(file));
    	
			while ((s = br.readLine())!=null) {
				   //System.out.println(s);
				   s = s.replace(".fa", "");
				   Matcher matcher = pattern.matcher( s );					
				   if (matcher.find()) {								
						String chrom = matcher.group(1);
						Integer left = Integer.parseInt(matcher.group(2));
						Integer right = Integer.parseInt(matcher.group(3));
						Float score = Float.parseFloat(matcher.group(4));
						if (!arrL.containsKey(chrom)) {
							arrL.put(chrom, new ArrayList <Integer>());
							arrR.put(chrom, new ArrayList <Integer>());
							scor.put(chrom, new ArrayList <Float>());
						}
						arrL.get(chrom).add(left);
						arrR.get(chrom).add(right);
						scor.get(chrom).add(score);
				   }	
			}
			br.close();
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}			
	}
	public static void main(String[] args) throws IOException {
		String usage = "java -Xmx1565m Summary -f <FindPeaks output file for ChIP> -c <FindPeaks output file for Control> -t <threshold on depth of DNA fragment coverage of peak> -v <ratio for filtering peaks using the control dataset>";
		if (args.length <4) {
			System.err.println("Too few arguments\n");
			System.err.println(usage);
			System.exit(1);
		}
		
				
		File chipFile = null;
		File controlFile = null;
		Float ratioForFiltering = 0f;
		Integer minimalCoverage = 0;
	
		int currentIndex = 0;	
		

		while (currentIndex<args.length) {
			//System.out.println(args[currentIndex]);	
			if (args[currentIndex].compareTo("-f")==0) {
				currentIndex++;
				chipFile = new File (args[currentIndex]);
				
			}			
			 
			else if (args[currentIndex].compareTo("-c")==0) {
				currentIndex++;
				controlFile = new File (args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-t")==0) {
				currentIndex++;
				minimalCoverage = Integer.parseInt(args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-v")==0) {
				currentIndex++;
				ratioForFiltering = Float.parseFloat(args[currentIndex]);
			}
			currentIndex++;
		}
		runFilterPeaks( chipFile,  controlFile,  ratioForFiltering,  minimalCoverage) ;
		
	}
	
}
