package micsa.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import components.ProgressMonitorMicsa;


import java.util.*;

/**
 * <p>Title: DESSolver Main Panel</p>
 * <p>Description: This class is the representation of the GUIs Main Panel</p>
  */

public class MainPanel extends JPanel implements ActionListener
{
  static ResourceBundle res = ResourceBundle.getBundle("micsa");

  
  File chipFile = null;//File with chipData  
  File controlFile = null;//File with control data 
  File outputDir = null; //output directory  
  File genomeDir = null;//directory with the genome sequences  
  Integer minLength = null;  //minimal length of DNA fragments if known
  Integer maxLength = null;  //maximal length of DNA fragments if known
  Integer meanLength = null;  //average length of DNA fragments if known
  Float ratioForFiltering = null;  //ratio for filtering peaks using Control Dataset
  Float minimalCoverage = null;  //minimal coverage by overlaping DNA fragments of potential binding site
  String aligner = res.getString("AL_MAQ64");  //aligner tool 
  Integer MaqQT = 30;  //Maq quality threshold
  File genomeBlackList = null; //file with genome positions to delete
  Float fpNumber = null;
  
  // a String array with all possible math Methods in it.
  public final String[] aligners = { res.getString("AL_MAQ64"),res.getString("AL_MAQ128"),res.getString("AL_MAPVIEW"),res.getString("AL_ELAND"),
		  res.getString("AL_ELAND_EXT"),res.getString("AL_BED"),res.getString("AL_GFF") };  
  
  // Here we declare all the Labels & TextFields
  private JTextField chIPfilename = new JTextField();
  private JTextField controlfilename = new JTextField();
  private JTextField genomeDirectory = new JTextField("\t");
  private JTextField outputDirectory = new JTextField();
  private JTextField blistFilename = new JTextField();
  private JComboBox  alignerComboBox = new JComboBox(aligners);
  private DecimalField  minLengthTextF = new DecimalField("", 5);
  private DecimalField  maxLengthTextF = new DecimalField("", 5);
  private DecimalField  meanLengthTextF = new DecimalField("", 5);
  private DecimalField  maqQTTextF = new DecimalField(MaqQT, 5);
  private DecimalField  ratioFilteringTextF = new DecimalField(2, 5);
  private DecimalField  minThreshTextF = new DecimalField(3, 5);
  private DecimalField  fpTextF = new DecimalField("", 5);
  

  private JLabel chipFileLabel = new JLabel(res.getString("LB_CHIPFILE")+" ", JLabel.RIGHT);
  private JLabel controlFileLabel = new JLabel(res.getString("LB_CONTROLFILE")+" ", JLabel.RIGHT);
  private JLabel genomeDirectoryLabel = new JLabel(res.getString("LB_GENOMEDIR")+" ", JLabel.RIGHT);
  private JLabel outputDirectoryLabel = new JLabel(res.getString("LB_OUTPUTDIR")+" ", JLabel.RIGHT);
  private JLabel blistFileLabel = new JLabel(res.getString("LB_BLACKLIST")+" ", JLabel.RIGHT);
  private JLabel alignerLabel = new JLabel(res.getString("LB_ALIGNMETHOD")+" ", JLabel.RIGHT);
  private JLabel minLengthLabel = new JLabel(res.getString("LB_MINLEN")+" ", JLabel.RIGHT);
  private JLabel maxLengthLabel = new JLabel(res.getString("LB_MAXLEN")+" ", JLabel.RIGHT);
  private JLabel meanLengthLabel = new JLabel(res.getString("LB_MEANLEN")+" ", JLabel.RIGHT);
  private JLabel maqQTLabel = new JLabel(res.getString("LB_MAQQT")+" ", JLabel.RIGHT);
  private JLabel ratioFilteringLabel = new JLabel(res.getString("LB_RATIO")+" ", JLabel.RIGHT);
  private JLabel setZeroLabel = new JLabel(res.getString("LB_SETZERO")+" ", JLabel.RIGHT);  
  private JLabel minThreshLabel = new JLabel(res.getString("LB_THRESHOLD")+" ", JLabel.RIGHT);
  private JLabel fpLabel = new JLabel(res.getString("LB_FALSEPOS")+" ", JLabel.RIGHT);
  private JLabel ifknLabel1 = new JLabel(res.getString("LB_IFKNOWN")+" ", JLabel.RIGHT);
  private JLabel ifknLabel2 = new JLabel(res.getString("LB_IFKNOWN")+" ", JLabel.RIGHT);
  private JLabel ifknLabel3 = new JLabel(res.getString("LB_IFKNOWN")+" ", JLabel.RIGHT);
  private JLabel ifknLabel4 = new JLabel(res.getString("LB_IFKNOWN")+" ", JLabel.RIGHT);
  private JLabel setonlyLabel = new JLabel(res.getString("LB_SETONLY")+" ", JLabel.RIGHT);
  private JLabel setonlyLabel2 = new JLabel(res.getString("LB_SETONLY")+" ", JLabel.RIGHT);

  
  
  //And Buttons
  private JButton chIPFileSetButton = new JButton(res.getString("BT_SET_CHIP_FILE"));
  private JButton controlFileSetButton = new JButton(res.getString("BT_SET_CONTROL_FILE"));
  private JButton genomeDirSetButton = new JButton(res.getString("BT_SET_GENOME_DIR"));
  private JButton outputDirSetButton = new JButton(res.getString("BT_SET_OUTPUT_DIR"));
  private JButton blistFileSetButton = new JButton(res.getString("BT_SET_BLACKLIST_FILE"));
  private JButton runButton = new JButton(res.getString("BT_CHECKPARAM"));
 
  //And Panel with monitor
  private JPanel panelMonitor ;
  
    public MainPanel()
  {
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
 
    // lets create a dispParamPanel
    JPanel dispParamPanel = new JPanel();
    dispParamPanel.setLayout(new BoxLayout(dispParamPanel, BoxLayout.Y_AXIS));     
    
      
    JPanel filesPanel = new JPanel();
    //filesPanel.setLayout(new BoxLayout(filesPanel, BoxLayout.Y_AXIS));
    filesPanel.setBorder(BorderFactory.createTitledBorder(res.getString("TXT_FILES")));
    filesPanel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();

    JPanel varPanel = new JPanel();
    varPanel.setLayout(new GridBagLayout());
    varPanel.setBorder(BorderFactory.createTitledBorder(res.getString("TXT_PARAMETERS")));
    
    
   // create TextFields for Files
    chipFileLabel.setLabelFor(chIPfilename);
    controlFileLabel.setLabelFor(controlfilename);
    genomeDirectoryLabel.setLabelFor(genomeDirectory);
    outputDirectoryLabel.setLabelFor(outputDirectory);

    c.fill = GridBagConstraints.HORIZONTAL;

    
    JPanel p = new JPanel();
    chIPFileSetButton.addActionListener(this);
    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
    p.add(chipFileLabel);
    chIPfilename.setEditable(false);
    chIPfilename.setBorder(null);
    p.add(chIPfilename);  
    c.fill = GridBagConstraints.HORIZONTAL;
    c.insets = new Insets(5,5,5,5);  // padding
    c.gridwidth = 2;

    c.weightx = 1.5;
    c.gridx = 0;
    c.gridy = 0;
    filesPanel.add(p, c);
    c.gridwidth = 1;

    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 0;
    filesPanel.add(chIPFileSetButton, c);      
   
    JPanel p2 = new JPanel();
    controlFileSetButton.addActionListener(this);
    p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
    p2.add(controlFileLabel);
    controlfilename.setEditable(false);
    controlfilename.setBorder(null);
    p2.add(controlfilename); 
    c.weightx = 1.5;
    c.gridwidth = 2;

    c.gridx = 0;
     c.gridy = 1;
     filesPanel.add(p2, c);
     c.gridwidth = 1;

     c.weightx = 0.5;
     c.gridx = 2;
     c.gridy = 1;
     filesPanel.add(controlFileSetButton, c);
    
    
    JPanel p3 = new JPanel();
    genomeDirSetButton.addActionListener(this);
    p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));
    p3.add(genomeDirectoryLabel);  
    genomeDirectory.setEditable(false);
    genomeDirectory.setBorder(null);  
    p3.add(genomeDirectory); 
  //  p3.add(setonlyLabel2);
    c.weightx = 0.5;
    c.gridx = 0;
    c.gridy = 4;
    filesPanel.add(p3, c);  
    c.weightx = 2.5;
    c.gridx = 1;
    c.gridy = 4;
    filesPanel.add(setonlyLabel2, c);  
    
    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 4;
    filesPanel.add(genomeDirSetButton, c);
   
     
    JPanel p4 = new JPanel();
    outputDirSetButton.addActionListener(this);
    p4.setLayout(new BoxLayout(p4, BoxLayout.X_AXIS));
    p4.add(outputDirectoryLabel);
    outputDirectory.setEditable(false);
    outputDirectory.setBorder(null); 
    p4.add(outputDirectory); 
    c.gridwidth = 2;

    c.weightx = 1.5;
    c.gridx = 0;
    c.gridy = 3;
    filesPanel.add(p4, c);
    c.gridwidth = 1;

    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 3;
    filesPanel.add(outputDirSetButton, c);  
    
    
    JPanel p14 = new JPanel();
    blistFileSetButton.addActionListener(this);
    p14.setLayout(new BoxLayout(p14, BoxLayout.X_AXIS));
    p14.add(blistFileLabel);
    blistFilename.setEditable(false);
    blistFilename.setBorder(null); 
    p14.add(blistFilename);p14.add(ifknLabel1);       
    c.gridwidth = 2;
    c.weightx = 1.5;
    c.gridx = 0;
    c.gridy = 2;
    filesPanel.add(p14, c);
    c.gridwidth = 1;
    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 2;
    filesPanel.add(blistFileSetButton, c);  
    
    /*
    JPanel p = new JPanel();
    chIPFileSetButton.addActionListener(this);
    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
    p.add(chipFileLabel);
    chIPfilename.setEditable(false);
    chIPfilename.setBorder(null);
    p.add(chIPfilename);
    p.add(chIPFileSetButton);
    filesPanel.add(p);
    
    
    
    JPanel p2 = new JPanel();
    controlFileSetButton.addActionListener(this);
    p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
    p2.add(controlFileLabel);
    controlfilename.setEditable(false);
    controlfilename.setBorder(null);
    p2.add(controlfilename);
    p2.add(controlFileSetButton);

 
   
   JPanel p3 = new JPanel();
   genomeDirSetButton.addActionListener(this);
   p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));
   p3.add(genomeDirectoryLabel);
  // filesPanel.add(p, BorderLayout.SOUTH);
   genomeDirectory.setEditable(false);
   genomeDirectory.setBorder(null);
  // p = new JPanel();
 //  p.setLayout(new GridLayout(2,1));
   p3.add(genomeDirectory);
   p3.add(genomeDirSetButton);
   filesPanel.add(p3);
    
   JPanel p4 = new JPanel();
   outputDirSetButton.addActionListener(this);
   p4.setLayout(new BoxLayout(p4, BoxLayout.X_AXIS));
   p4.add(outputDirectoryLabel);
   outputDirectory.setEditable(false);
   outputDirectory.setBorder(null); 
   p4.add(outputDirectory);
   p4.add(outputDirSetButton);
   filesPanel.add(p4);

   JPanel p14 = new JPanel();
   blistFileSetButton.addActionListener(this);
   p14.setLayout(new BoxLayout(p14, BoxLayout.X_AXIS));
   p14.add(blistFileLabel);
   blistFilename.setEditable(false);
   blistFilename.setBorder(null); 
   p14.add(blistFilename);p14.add(ifknLabel1);
   p14.add(blistFileSetButton);
   filesPanel.add(p14);   
   
   chIPFileSetButton.setMaximumSize(new Dimension( 60, 25 ) );
   controlFileSetButton.setMaximumSize( new Dimension( 60, 25 ) );
   genomeDirSetButton.setMaximumSize( new Dimension( 60, 25 ) );
   outputDirSetButton.setMaximumSize( new Dimension( 60, 25 ) );
   blistFileSetButton.setMaximumSize( new Dimension( 60, 25 ) );

   chIPFileSetButton.setMinimumSize(new Dimension( 60, 25 ) );
   controlFileSetButton.setMinimumSize( new Dimension( 60, 25 ) );
   genomeDirSetButton.setMinimumSize( new Dimension( 60, 25 ) );
   outputDirSetButton.setMinimumSize( new Dimension( 60, 25 ) );
   blistFileSetButton.setMinimumSize( new Dimension( 60, 25 ) );
 */
  
 
   // create TextFields for parameters
    
   JPanel p5 = new JPanel();
   alignerComboBox.addActionListener(this);
   p5.setLayout(new BoxLayout(p5, BoxLayout.X_AXIS));
   p5.add(alignerLabel);
   //p5.add(alignerComboBox);
  // varPanel.add(p5);
   c.gridwidth = 2;
   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 0;
   varPanel.add(p5, c);
   c.gridwidth = 1;
   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 0;
   varPanel.add(alignerComboBox, c);  
   
   
   JPanel p16 = new JPanel();
   maqQTLabel.setLabelFor(maqQTTextF);
   maqQTTextF.addActionListener(this);
   p16.setLayout(new BoxLayout(p16, BoxLayout.X_AXIS));
   p16.add(maqQTLabel);
   //p16.add(maqQTTextF);
   c.gridwidth = 2;

   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 1;
   varPanel.add(p16,c);
   c.gridwidth = 1;

   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 1;
   varPanel.add(maqQTTextF, c);  
   
   

   JPanel p6 = new JPanel();
   minLengthLabel.setLabelFor(minLengthTextF);
   minLengthTextF.addActionListener(this);
   p6.setLayout(new BoxLayout(p6, BoxLayout.X_AXIS));
   p6.add(minLengthLabel);p6.add(ifknLabel2);
  // p6.add(minLengthTextF);
  // varPanel.add(p6);
   c.gridwidth = 2;

   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 2;
   varPanel.add(p6,c);
   c.gridwidth = 1;

   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 2;
   varPanel.add(minLengthTextF, c);     
     
   
   JPanel p7 = new JPanel();
   maxLengthLabel.setLabelFor(maxLengthTextF);
   maxLengthTextF.addActionListener(this);
   p7.setLayout(new BoxLayout(p7, BoxLayout.X_AXIS));
   p7.add(maxLengthLabel);p7.add(ifknLabel3);
  // p7.add(maxLengthTextF);
  // varPanel.add(p7);
   c.gridwidth = 2;

   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 3;
   varPanel.add(p7,c);
   c.gridwidth = 1;

   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 3;
   varPanel.add(maxLengthTextF, c);     
   
   JPanel p8 = new JPanel();
   meanLengthLabel.setLabelFor(meanLengthTextF);
   meanLengthTextF.addActionListener(this);
   p8.setLayout(new BoxLayout(p8, BoxLayout.X_AXIS));
   p8.add(meanLengthLabel);p8.add(ifknLabel4);
  // p8.add(meanLengthTextF);
   c.gridwidth = 2;

   
   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 4;
   varPanel.add(p8,c);
   c.gridwidth = 1;

   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 4;
   varPanel.add(meanLengthTextF, c);     
   
  // varPanel.add(p8);  
   
   JPanel p19 = new JPanel();
   minThreshLabel.setLabelFor(minThreshTextF);
   minThreshTextF.addActionListener(this);
   p19.setLayout(new BoxLayout(p19, BoxLayout.X_AXIS));
   p19.add(minThreshLabel);
   //p19.add(minThreshTextF);
   //varPanel.add(p19);  
   c.gridwidth = 2;

   c.weightx = 1.5;
   c.gridx = 0;
   c.gridy = 5;
   varPanel.add(p19,c);
   c.gridwidth = 1;

   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 5;
   varPanel.add(minThreshTextF, c);  
   
   JPanel p18 = new JPanel();
   ratioFilteringLabel.setLabelFor(ratioFilteringTextF);
   ratioFilteringTextF.addActionListener(this);
   p18.setLayout(new BoxLayout(p18, BoxLayout.X_AXIS));
   p18.add(ratioFilteringLabel);
  // p18.add(setZeroLabel);   
  // p18.add(ratioFilteringTextF);
   //varPanel.add(p18);
   
   c.weightx = 0.5;
   c.gridx = 0;
   c.gridy = 6;
   varPanel.add(p18,c);
   
   c.weightx = 0.5;
   c.gridx = 1;
   c.gridy = 6;
   varPanel.add(setZeroLabel,c);
   
   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 6;
   varPanel.add(ratioFilteringTextF, c);  
   
   JPanel p20 = new JPanel();
   fpLabel.setLabelFor(fpTextF);
   fpTextF.addActionListener(this);
   p20.setLayout(new BoxLayout(p20, BoxLayout.X_AXIS));
   p20.add(fpLabel);//p20.add(setonlyLabel);
   //p20.add(fpTextF);
   //varPanel.add(p20);    
   
   c.weightx = 0.5;
   c.gridx = 0;
   c.gridy = 7;
   varPanel.add(p20,c);
   c.weightx = 2.5;
   c.gridx = 1;
   c.gridy = 7;
   varPanel.add(setonlyLabel,c);
   c.weightx = 0.5;
   c.gridx = 2;
   c.gridy = 7;
   varPanel.add(fpTextF, c);  
  
  // int maxwidth = (int)controlFileSetButton.getMaximumSize().getWidth();
  // int minheight = (int)controlFileSetButton.getMinimumSize().getHeight();
  // buttonPanel.setMaximumSize(new Dimension(maxwidth, minheight));
  // chIPFileSetButton.setSize(maxwidth, minheight);
   //genomeDirSetButton 
   //outputDirSetButton 
   //chIPFileSetButton.repaint();
   
    // now lets add all the Labels & TextFields to it.
   
    dispParamPanel.add(filesPanel);
    dispParamPanel.add(varPanel);

    // create the buttonPanel with controls
    JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    buttonPanel.add(runButton);
    runButton.addActionListener(this);
    
    
    //create panel processRun;     

       
   // int maxwidth = (int)buttonPanel.getMaximumSize().getWidth();
   // int minheight = (int)buttonPanel.getMinimumSize().getHeight();
  //  buttonPanel.setMaximumSize(new Dimension(maxwidth, minheight));
    add(dispParamPanel);
    add(buttonPanel);  
      
     
  }

  // lets define the ActionListener Method
  public void actionPerformed(ActionEvent e)
  {
	  
	if (e.getActionCommand().equals(res.getString("BT_CHECKPARAM"))){
		//TODO read all text fields
		MaqQT =  ((Double)maqQTTextF.getValue()).intValue(); 
		ratioForFiltering = new Float(ratioFilteringTextF.getValue()); 
		minimalCoverage = new Float(minThreshTextF.getValue()); 
		if (minLengthTextF.getText().length()>0) {
			minLength = Math.round(Float.parseFloat(minLengthTextF.getText()));
		}
		if (maxLengthTextF.getText().length()>0) {
			maxLength = Math.round(Float.parseFloat(maxLengthTextF.getText()));
		}
		if (meanLengthTextF.getText().length()>0) {
			meanLength = Math.round(Float.parseFloat(meanLengthTextF.getText()));
		}			
		if (fpTextF.getText().length()>0) {
			fpNumber = Float.parseFloat(fpTextF.getText());
		}				
		if (checkParameters())  {
			runButton.setVisible(false);
			runApplication();			
		}
		else {
		      String message = "You did not set all mandatory parameters!\nPlease check!\n" +"\n\n";
		      JOptionPane.showMessageDialog(null, message, res.getString("TXT_APPNAME")+" v"+MICSAmainPanel.VERSION, JOptionPane.INFORMATION_MESSAGE);
		}			
	}
	if (e.getActionCommand().equals(res.getString("BT_SET_CHIP_FILE"))){		
		chipFile = getFileNameFromChooser (chIPfilename);
	}
	if (e.getActionCommand().equals(res.getString("BT_SET_CONTROL_FILE"))){		
		controlFile = getFileNameFromChooser (controlfilename);
	}	
	if (e.getActionCommand().equals(res.getString("BT_SET_GENOME_DIR"))){		
		genomeDir = getDirFromChooser (genomeDirectory);
	}	
	if (e.getActionCommand().equals(res.getString("BT_SET_OUTPUT_DIR"))){		
		outputDir = getDirFromChooser (outputDirectory);
	}
	
	if (e.getActionCommand().equals(res.getString("BT_SET_BLACKLIST_FILE"))){		
		genomeBlackList = getFileNameFromChooser (blistFilename);
	}	
	if(e.getSource().equals(alignerComboBox))
    {     
		aligner = (String)alignerComboBox.getSelectedItem();      
		
		if (!aligner.contains("Ma")) {
			maqQTTextF.setVisible(false);
			maqQTLabel.setVisible(false);

		}
		else {
			maqQTTextF.setVisible(true);
			maqQTLabel.setVisible(true);
		}
    }
	
 }
  
  
  private void runApplication() {
	  
	  JComponent newContentPane = new ProgressMonitorMicsa(aligner,chipFile,controlFile,genomeDir,outputDir,genomeBlackList,minLength,maxLength,meanLength,MaqQT,ratioForFiltering,Math.round(minimalCoverage),fpNumber);
      newContentPane.setOpaque(true); //content panes must be opaque      
      panelMonitor = new JPanel();      
      panelMonitor.add(newContentPane);
      add(panelMonitor);  
}

private boolean checkParameters() {
	//TODO

	if (chipFile==null)
		return false;
	if (controlFile==null)
		return false;
	if (outputDir==null)
		return false;
	else { //check if it exists
		if (!outputDir.exists()) { //create it
			 boolean success = outputDir.mkdir();
			 if (!success)
			 	return false;
		}		
	}
	//if (genomeDir==null)
	//	return false; 
	return true;
}

public File getFileNameFromChooser (JTextField jtf) {
	  
	    JFileChooser c = new JFileChooser( );
	    int rVal = c.showOpenDialog(this);
	    if (rVal == JFileChooser.CANCEL_OPTION) {
	    	jtf.setText("");
	    	return null;
	    }
	    if (rVal == JFileChooser.APPROVE_OPTION) {
	    	jtf.setText(c.getSelectedFile().getName());
	    	return c.getSelectedFile();
	    }
	    	    
	    return null;	  
  }
  public File getDirFromChooser (JTextField jtf) {
	  
	    JFileChooser c = new JFileChooser( );
	    c.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

	    int rVal = c.showOpenDialog(this);
	    if (rVal == JFileChooser.CANCEL_OPTION) {
	    	jtf.setText("");
	    	return null;
	    }
	    if (rVal == JFileChooser.APPROVE_OPTION) {
	    	jtf.setText(c.getSelectedFile().getName());
	    	return c.getSelectedFile();
	    }
	    	    
	    return null;	  
}
  /* 
  public void loadFile ( ) {
	    JFileChooser chooser = new JFileChooser( );
	    int result = chooser.showOpenDialog(this);
	    if (result == JFileChooser.CANCEL_OPTION) return;
	    try {
	      File file = chooser.getSelectedFile( );	  
	      
	      FileInputStream fis = null;
	      BufferedInputStream bis = null;
	      DataInputStream dis = null;
	      StringBuilder newFunction = new StringBuilder(); 

	      try {
	        fis = new FileInputStream(file);

	        // Here BufferedInputStream is added for fast reading.
	        bis = new BufferedInputStream(fis);
	        dis = new DataInputStream(bis);

	        // dis.available() returns 0 if the file does not have more lines.
	        while (dis.available() != 0) {

	        // this statement reads the line from the file and print it to
	          // the console.
	        	newFunction.append(dis.readLine()+"\n");	        	
	        }

	        // dispose all the resources after using them.
	        fis.close();
	        bis.close();
	        dis.close();

	      } catch (FileNotFoundException e) {
	        e.printStackTrace();
	      } catch (IOException e) {
	        e.printStackTrace();
	      }
	      
	      //System.out.println(file.getName()+"\n"+file.getPath()+"\n"+file.getParent());
	  	
	      
	      
	      //setEnvironment(xMin, xMax, yMin, yMax, incr, newFunction.toString()); depreciated
	      System.out.println("setEnvironment set");
	    }
	    catch (Exception e) {
	    	System.out.println("Could not load file: " + e);	
	    	//textPane.setText("Could not load file: " + e);
	    }
	    
	  }
	 
  public void saveFile( ) {
	    JFileChooser chooser = new JFileChooser( );
	    chooser.showSaveDialog(this);
	    // Save file data...
  }
  public void checkParameterPanel(List parameters){
	  if (parameters==null){
		  System.out.println("Parameters = null");
		  return;
	  } 
  }
  */  
   
  
}