package ru.genetika.pwm.scanner;

import java.util.Iterator;

import meme.Motif;

import org.arabidopsis.ahocorasick.AhoCorasick;
import org.arabidopsis.ahocorasick.SearchResult;

import ru.genetika.common.ByteSequence;
//import ru.genetika.common.CharSequence;
import ru.genetika.pwm.Pwm;
import ru.genetika.pwm.utilities.IPwmWordListener;
import ru.genetika.pwm.utilities.PwmWordGenerator;
import ru.genetika.pwm.utilities.Word;

public class PwmScannerAhoC extends PwmScanner implements IPwmWordListener {

	byte[] sequence = null;
	boolean changed = false;
	AhoCorasick ahoC = null;
	
	@Override
	public void setMinThreshold(Float t) {
		super.setMinThreshold(t);
		changed = true;
	}
	
	@Override
	public void setPwm(Pwm pwm) {
		super.setPwm(pwm);
		changed = true;
	}
	
	public void setSequence(ByteSequence sequence) {
		this.sequence = sequence.getSequence();
	}
	
	@Override
	public void scan() {
		if(changed)
		{
			ahoC = new AhoCorasick();
			PwmWordGenerator wordGenerator = new PwmWordGenerator(pwmFw);
			wordGenerator.setThreshold(minThreshold);
			wordGenerator.setListener(this);
			wordGenerator.generateWords();
			ahoC.prepare();
			changed = false;
		}
		Iterator iter = ahoC.search(sequence);
		while(iter.hasNext())
		{
			SearchResult result = (SearchResult)iter.next();
			resultListener.receiveScore(result.getLastIndex() - pwmFw.size(), ((float[])result.getOutputs().get(0))[0], ((float[])result.getOutputs().get(0))[1]);
		}
	}
	
	public void scanToProb(Motif motif) {
		if(changed)
		{
			ahoC = new AhoCorasick();
			PwmWordGenerator wordGenerator = new PwmWordGenerator(pwmFw);
			wordGenerator.setThreshold(minThreshold);
			wordGenerator.setListener(this);
			wordGenerator.generateWords();
			ahoC.prepare();
			changed = false;
		}
		//float precision = motif.getPrecision();
		Iterator iter = ahoC.search(sequence);
		while(iter.hasNext())
		{
			SearchResult result = (SearchResult)iter.next();
			float score = Math.max(((float[])result.getOutputs().get(0))[0],((float[])result.getOutputs().get(0))[1]);
			//int position = result.getLastIndex() - pwmFw.size();
			//resultListener.receiveScore(position, score, score);
			float thresh = (float) Math.floor(score); //use ceil?
				//((float)((int)(precision*score)))/precision;						
			motif.addOcc(thresh);	// attention, it may be better to use "motif.addOcc(thresh+1f/precision);"			
		}
	}
	

	public void receiveWord(Word word) {
		ahoC.add(word.getWord().getBytes(), new float[]{word.getScoreFw(), word.getScoreRev()});
	}	
	
	protected void finalise() {
		try {
			super.finalize();
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
}
