package ru.genetika.pwm.scanner;

import ru.genetika.common.CharSequence;

import ru.genetika.pwm.Pwm;
import meme.*;

public class PwmScannerNaive extends PwmScanner {

	char[] sequence = null;
	protected Pwm pwmRv = null;
	
	public PwmScannerNaive()
	{
	}
	
	@Override
	public void setPwm(Pwm pwm) {
		super.setPwm(pwm);
		pwmRv = pwmFw.getReverseComplementMatrix();
	}

	public void setSequence(CharSequence sequence) {
		this.sequence = sequence.getSequence();
	}
	
	public void scan() {
		for(int i = 0; i + pwmFw.size() <= sequence.length && sequence[i + pwmFw.size() - 1] != '\0' ; ++i)
		{
			float scoreFw = 0;
			float scoreRv = 0;
			for(int j = 0; j < pwmFw.size(); ++j)
			{
				scoreFw += pwmFw.get(j, sequence[j+i]);
				scoreRv += pwmRv.get(j, sequence[j+i]);
			}
			if(scoreFw >= minThreshold || scoreRv >= minThreshold)
			{
				resultListener.receiveScore(i, scoreFw, scoreRv);
			}
		}
	}
	public Sample scanToSample() {
		Sample sample = new Sample();
		
		for(int i = 0; i + pwmFw.size() <= sequence.length && sequence[i + pwmFw.size() - 1] != '\0' ; ++i)
		{
			float scoreFw = 0;
			float scoreRv = 0;
			for(int j = 0; j < pwmFw.size(); ++j)
			{
				scoreFw += pwmFw.get(j, sequence[j+i]);
				scoreRv += pwmRv.get(j, sequence[j+i]);
			}
			if(scoreFw >= minThreshold || scoreRv >= minThreshold)
			{
				//resultListener.receiveScore(i, scoreFw, scoreRv);
				Site st ;
				if (scoreFw > scoreRv) {
					st = new Site(i, scoreFw, true);
				}
				else {
					st = new Site(i, scoreRv, false);
				}
				sample.incrNSites(st);
			}
		}
		if (pwmFw.prob()!=null)
			sample.calculatePvalue(sequence.length - pwmFw.size() + 1, pwmFw.prob());
		return sample;
	}
	public int scanToProb(Motif motif) {  /* scan sequence and return number of ACGT letter read. Changes Motif.probs values */
			
		int textLength = 0;
		float precision = motif.getPrecision();
		int matrixLength = pwmFw.size();
		int chrLength = sequence.length;
		//for(int i = 0; i + matrixLength <= chrLength && sequence[i + matrixLength - 1] != '\0' ; ++i)

		for(int i = 0; i + matrixLength <= chrLength-100 ; ++i)
		{
			if (sequence[i] != 'N') {			
				textLength++;
				float scoreFw = 0;
				float scoreRv = 0;
				for(int j = 0; j < matrixLength; ++j)
				{
					scoreFw += pwmFw.get(j, sequence[j+i]);
					scoreRv += pwmRv.get(j, sequence[j+i]);
				}
				if(scoreFw >= minThreshold || scoreRv >= minThreshold)
				{
					float score = Math.max(scoreFw,scoreRv);
					float thresh = 	(float)Math.floor(score);						
					motif.addOcc(thresh);	// attention, it may be better to use "motif.addOcc(thresh+1f/precision);"				
				}			
			}	
		}
		return textLength;
	}

	public void setPwmProb(double prob) {
		pwmFw.setProb(prob);		
	}
}
